{********************************************************************************}
{ Othello 1.0 - Juego del Othello bajo ms-dos.                                   }
{ Copyright (C) 2005  Gorka Surez Garca                                        }
{                                                                                }
{ IAJuego.pas is part of "Othello".                                              }
{                                                                                }
{ "Othello" is free software; you can redistribute it and/or                     }
{ modify it under the terms of the GNU General Public License                    }
{ as published by the Free Software Foundation; either version 2                 }
{ of the License, or (at your option) any later version.                         }
{                                                                                }
{ "Othello" is distributed in the hope that it will be useful,                   }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                 }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  }
{ GNU General Public License for more details.                                   }
{                                                                                }
{ You should have received a copy of the GNU General Public License              }
{ along with "Othello"; if not, write to the Free Software                       }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    }
{********************************************************************************}
unit IAJuego;
{********************************************************************************}
interface {parte publica}
uses Lesslib, Nucleo, LogMovs, crt;
{********************************************************************************}
{ Funciones para controlar la inteligencia artificial del juego.                 }
{********************************************************************************}
function  DarNumeroFichasBando (var Nucleo: tNucleo; color: byte): integer;
procedure MaquinaMueve         (var Nucleo: tNucleo);

function Maximizador (var Nucleo: tNucleo; color, profundidad: byte;
                      alpha, beta: integer): integer;
function Minimizador (var Nucleo: tNucleo; color, profundidad: byte;
                      alpha, beta: integer): integer;


{********************************************************************************}
implementation {parte privada}
{********************************************************************************}


{********************************************************************************}
{ Constantes relacionadas con la IA del juego.                                   }
{********************************************************************************}
const
   ALPHA_INICIAL = -32768;
   BETA_INICIAL  =  32767;

{********************************************************************************}
{ Funcin que nos devuelve el nmero de fichas de un bando.                      }
{********************************************************************************}
function DarNumeroFichasBando (var Nucleo: tNucleo; color: byte): integer;
var
   c, f: byte;
   result: integer;
begin
   {-----------------------------------------------------------------------------}
   { Inicializamos el contador.                                                  }
   {-----------------------------------------------------------------------------}
   result := 0;

   {-----------------------------------------------------------------------------}
   { Y recorremos el tablero buscando fichas del mismo color.                    }
   {-----------------------------------------------------------------------------}
   for f := 1 to Nucleo.Juego.MaxFilas do
      for c := 1 to Nucleo.Juego.MaxColum do
         if Nucleo.Juego.Fichas[f, c] = color then
            result := result + 1;

   {-----------------------------------------------------------------------------}
   { Devolvemos el resultado obtenido.                                           }
   {-----------------------------------------------------------------------------}
   DarNumeroFichasBando := result;
end;

{********************************************************************************}
{ Funcin que nos devuelve una evaluacin simple del tablero actual.             }
{********************************************************************************}
function EvaluarTablero (var Nucleo: tNucleo): integer;
var
   numbla, numneg: integer;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver cuantas fichas hay en cada bando.               }
   {-----------------------------------------------------------------------------}
   numbla := DarNumeroFichasBando(Nucleo, FICHA_BLANCA);
   numneg := DarNumeroFichasBando(Nucleo, FICHA_NEGRA);

   {-----------------------------------------------------------------------------}
   { Y la evaluacin ser el nmero de negras, menos las blancas.                }
   {-----------------------------------------------------------------------------}
   EvaluarTablero := numneg - numbla;
end;

{********************************************************************************}
{ Funcin que nos devuelve el bando que le tocar mover en el siguiente turno.   }
{********************************************************************************}
function DarSigTurno (color: byte): byte;
begin
   {-----------------------------------------------------------------------------}
   { Si es el turno de las blancas, el prximo ser el de las negras.            }
   {-----------------------------------------------------------------------------}
   if color = TURNO_BLANCAS then
      DarSigTurno := TURNO_NEGRAS
   {-----------------------------------------------------------------------------}
   { Sino es el turno de las negras, y el prximo ser el de las blancas.        }
   {-----------------------------------------------------------------------------}
   else
      DarSigTurno := TURNO_BLANCAS;
end;

{********************************************************************************}
{ Funcin que sirve para escoger la mejor jugada para la mquina.                }
{********************************************************************************}
function Maximizador (var Nucleo: tNucleo; color, profundidad: byte;
                      alpha, beta: integer): integer;
var
   c, f: byte;
   aux: tTablero;
   valormax, valoract: integer;
   salir: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Determinamos si hay bloqueo, de haberlo "terminara" la partida.            }
   {-----------------------------------------------------------------------------}
   CalcularBloqueo(Nucleo);

   {-----------------------------------------------------------------------------}
   { Indicamos que el valor mximo todava no est inicializado.                 }
   {-----------------------------------------------------------------------------}
   valormax := ALPHA_INICIAL;

   {-----------------------------------------------------------------------------}
   { Si hemos sobrepasado la profundidad del "arbol" de bsqueda, o si la        }
   { partida ha terminado, devolvemos la evaluacin del tablero actual.          }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Dificultad < profundidad) or
      (Nucleo.Juego.BlanBloq and Nucleo.Juego.NegrBloq) then begin
      Maximizador := EvaluarTablero(Nucleo);
      
   {-----------------------------------------------------------------------------}
   { Sino echamos un vistazo a todas las posibles jugadas.                       }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Salvamos el tablero y ponemos a falso el flag de salir del bucle.        }
      {--------------------------------------------------------------------------}
      aux   := Nucleo.Juego.Fichas;
      salir := false;

      {--------------------------------------------------------------------------}
      { Recorremos todo el tablero probando casilla por casilla, a fin de        }
      { encontrar todas las posibles jugadas del turno actual.                   }
      {--------------------------------------------------------------------------}
      for f := 1 to Nucleo.Juego.MaxFilas do begin
         for c := 1 to Nucleo.Juego.MaxColum do begin
            if ValidarMovimiento(Nucleo, color, c, f) then begin
               {-----------------------------------------------------------------}
               { Pedimos el valor mnimo de las siguientes jugadas.              }
               {-----------------------------------------------------------------}
               valoract := Minimizador(Nucleo, DarSigTurno(color),
                                       profundidad + 1, alpha, beta);

               {-----------------------------------------------------------------}
               { Restauramos el tablero.                                         }
               {-----------------------------------------------------------------}
               Nucleo.Juego.Fichas := aux;

               {-----------------------------------------------------------------}
               { Y echamos un vistazo a la situacin en s. Si est sin          }
               { inicializar, o si el valor mximo actual es menor que el que    }
               { acabamos de conseguir, entonces actualizamos los valores. Y     }
               { adems cambiamos el valor de alpha, que es la cota inferior     }
               { que puede asignarse en ltimo termino al nodo maximizante.      }
               {-----------------------------------------------------------------}
               if (valormax < valoract) then begin
                  valormax := valoract;

                  alpha := valoract;
               end;

               {-----------------------------------------------------------------}
               { Si la cota inferior del mximo, es mayor que la cota superior   }
               { del mnimo, dejamos de comprobar los siguientes terminos.       }
               {-----------------------------------------------------------------}
               if (alpha > beta) then begin
                  salir := true;
                  break;
               end;
            end;
         end;

         if salir then break;
      end;

      {--------------------------------------------------------------------------}
      { Devolvemos el valor mximo obtenido.                                     }
      {--------------------------------------------------------------------------}
      Maximizador := valormax;
   end;
end;

{********************************************************************************}
{ Funcin que sirve para escoger la peor jugada para la mquina.                 }
{********************************************************************************}
function Minimizador (var Nucleo: tNucleo; color, profundidad: byte;
                      alpha, beta: integer): integer;
var
   c, f: byte;
   aux: tTablero;
   valormin, valoract: integer;
   salir: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Determinamos si hay bloqueo, de haberlo "terminara" la partida.            }
   {-----------------------------------------------------------------------------}
   CalcularBloqueo(Nucleo);

   {-----------------------------------------------------------------------------}
   { Indicamos que el valor mnimo todava no est inicializado.                 }
   {-----------------------------------------------------------------------------}
   valormin := BETA_INICIAL;

   {-----------------------------------------------------------------------------}
   { Si hemos sobrepasado la profundidad del "arbol" de bsqueda, o si la        }
   { partida ha terminado, devolvemos la evaluacin del tablero actual.          }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Dificultad < profundidad) or
      (Nucleo.Juego.BlanBloq and Nucleo.Juego.NegrBloq) then begin
      Minimizador := EvaluarTablero(Nucleo);
      
   {-----------------------------------------------------------------------------}
   { Sino echamos un vistazo a todas las posibles jugadas.                       }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Salvamos el tablero y ponemos a falso el flag de salir del bucle.        }
      {--------------------------------------------------------------------------}
      aux   := Nucleo.Juego.Fichas;
      salir := false;
      
      {--------------------------------------------------------------------------}
      { Recorremos todo el tablero probando casilla por casilla, a fin de        }
      { encontrar todas las posibles jugadas del turno actual.                   }
      {--------------------------------------------------------------------------}
      for f := 1 to Nucleo.Juego.MaxFilas do begin
         for c := 1 to Nucleo.Juego.MaxColum do begin
            if ValidarMovimiento(Nucleo, color, c, f) then begin
               {-----------------------------------------------------------------}
               { Pedimos el valor mnimo de las siguientes jugadas.              }
               {-----------------------------------------------------------------}
               valoract := Maximizador(Nucleo, DarSigTurno(color),
                                       profundidad + 1, alpha, beta);

               {-----------------------------------------------------------------}
               { Restauramos el tablero.                                         }
               {-----------------------------------------------------------------}
               Nucleo.Juego.Fichas := aux;

               {-----------------------------------------------------------------}
               { Y echamos un vistazo a la situacin en s. Si est sin          }
               { inicializar, o si el valor mnimo actual es mayor que el que    }
               { acabamos de conseguir, entonces actualizamos los valores. Y     }
               { adems cambiamos el valor de beta, que es la cota superior      }
               { que puede asignarse en ltimo termino al nodo minimizante.      }
               {-----------------------------------------------------------------}
               if (valormin > valoract) then begin
                  valormin := valoract;

                  beta := valoract;
               end;

               {-----------------------------------------------------------------}
               { Si la cota inferior del mximo, es mayor que la cota superior   }
               { del mnimo, dejamos de comprobar los siguientes terminos.       }
               {-----------------------------------------------------------------}
               if (alpha > beta) then begin
                  salir := true;
                  break;
               end;
            end;
         end;

         if salir then break;
      end;

      {--------------------------------------------------------------------------}
      { Devolvemos el valor mnimo obtenido.                                     }
      {--------------------------------------------------------------------------}
      Minimizador := valormin;
   end;
end;

{********************************************************************************}
{ Funcin que sirve para realizar el algoritmo del minimax para la IA.           }
{********************************************************************************}
function Minimax (var Nucleo: tNucleo; color, profundidad: byte;
                  alpha, beta: integer; var x, y: byte): integer;
var
   c, f: byte;
   aux: tTablero;
   valormax, valoract: integer;
begin
   {-----------------------------------------------------------------------------}
   { Determinamos si hay bloqueo, de haberlo "terminara" la partida.            }
   {-----------------------------------------------------------------------------}
   CalcularBloqueo(Nucleo);

   {-----------------------------------------------------------------------------}
   { Inicializamos las coordenadas a devolver con 0, 0.                          }
   {-----------------------------------------------------------------------------}
   x := 0;
   y := 0;

   {-----------------------------------------------------------------------------}
   { Si hemos sobrepasado la profundidad del "arbol" de bsqueda, o si la        }
   { partida ha terminado, devolvemos la evaluacin del tablero actual.          }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Dificultad < profundidad) or
      (Nucleo.Juego.BlanBloq and Nucleo.Juego.NegrBloq) then begin
      Minimax := EvaluarTablero(Nucleo);
      
   {-----------------------------------------------------------------------------}
   { Sino echamos un vistazo a todas las posibles jugadas.                       }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Salvamos el tablero y ponemos a falso el flag de salir del bucle.        }
      {--------------------------------------------------------------------------}
      aux   := Nucleo.Juego.Fichas;

      {--------------------------------------------------------------------------}
      { Recorremos todo el tablero probando casilla por casilla, a fin de        }
      { encontrar todas las posibles jugadas del turno actual.                   }
      {--------------------------------------------------------------------------}
      for f := 1 to Nucleo.Juego.MaxFilas do begin
         for c := 1 to Nucleo.Juego.MaxColum do begin
            if ValidarMovimiento(Nucleo, color, c, f) then begin
               {-----------------------------------------------------------------}
               { Pedimos el valor mnimo de las siguientes jugadas.              }
               {-----------------------------------------------------------------}
               valoract := Minimizador(Nucleo, DarSigTurno(color),
                                       profundidad + 1, alpha, beta);

               {-----------------------------------------------------------------}
               { Restauramos el tablero.                                         }
               {-----------------------------------------------------------------}
               Nucleo.Juego.Fichas := aux;

               {-----------------------------------------------------------------}
               { Y echamos un vistazo a la situacin en s. Si est sin          }
               { inicializar, o si el valor mximo actual es menor que el que    }
               { acabamos de conseguir, entonces actualizamos los valores. Y     }
               { adems cambiamos el valor de alpha, que es la cota inferior     }
               { que puede asignarse en ltimo termino al nodo maximizante.      }
               {-----------------------------------------------------------------}
               if ((x = 0) and (y = 0)) or (valormax < valoract) then begin
                  x := c;
                  y := f;

                  valormax := valoract;

                  alpha := valoract;
               end;
            end;
         end;
      end;

      {--------------------------------------------------------------------------}
      { Devolvemos el valor mximo obtenido.                                     }
      {--------------------------------------------------------------------------}
      Minimax := valormax;
   end;
end;

{********************************************************************************}
{ Funcin que gestiona los movimientos de la mquina.                            }
{********************************************************************************}
procedure MaquinaMueve (var Nucleo: tNucleo);
var
   c, f: byte;
begin
   {-----------------------------------------------------------------------------}
   { Buscamos la mejor coordenada posible para tirar con un mini-max.            }
   {-----------------------------------------------------------------------------}
   Minimax(Nucleo, TURNO_NEGRAS, 0, ALPHA_INICIAL, BETA_INICIAL, c, f);
   
   {-----------------------------------------------------------------------------}
   { Y lanzamos el movimiento que hemos determinado el mejor.                    }
   {-----------------------------------------------------------------------------}
   ValidarMovimiento(Nucleo, TURNO_NEGRAS, c, f);
end;

{********************************************************************************}
end.
{********************************************************************************}