{********************************************************************************}
{ Othello 1.0 - Juego del Othello bajo ms-dos.                                   }
{ Copyright (C) 2005  Gorka Surez Garca                                        }
{                                                                                }
{ LogMovs.pas is part of "Othello".                                              }
{                                                                                }
{ "Othello" is free software; you can redistribute it and/or                     }
{ modify it under the terms of the GNU General Public License                    }
{ as published by the Free Software Foundation; either version 2                 }
{ of the License, or (at your option) any later version.                         }
{                                                                                }
{ "Othello" is distributed in the hope that it will be useful,                   }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                 }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  }
{ GNU General Public License for more details.                                   }
{                                                                                }
{ You should have received a copy of the GNU General Public License              }
{ along with "Othello"; if not, write to the Free Software                       }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    }
{********************************************************************************}
unit LogMovs;
{********************************************************************************}
interface {parte publica}
uses Lesslib, Nucleo, crt;
{********************************************************************************}
{ Funciones para controlar la lgica de los movimientos del juego.               }
{********************************************************************************}
procedure CalcularBloqueo   (var Nucleo: tNucleo);
function  ValidarMovimiento (var Nucleo: tNucleo; color, c, f: byte): boolean;


{********************************************************************************}
implementation {parte privada}
{********************************************************************************}


{********************************************************************************}
{ Funcin que calcula si hay bloqueo para los dos bandos.                        }
{********************************************************************************}
procedure CalcularBloqueo (var Nucleo: tNucleo);
var
   color, c, f: byte;
   resultado: boolean;
   aux: tTablero;
begin
   {-----------------------------------------------------------------------------}
   { Salvamos el tablero en una variable auxiliar, para rescatarlo al mover.     }
   {-----------------------------------------------------------------------------}
   aux := Nucleo.Juego.Fichas;

   {-----------------------------------------------------------------------------}
   { Recorremos los dos bandos, el blanco y el negro.                            }
   {-----------------------------------------------------------------------------}
   for color := TURNO_BLANCAS to TURNO_NEGRAS do begin
      {--------------------------------------------------------------------------}
      { Iniciamos el resultado presuponiendo que s va a haber bloqueo.          }
      {--------------------------------------------------------------------------}
      resultado := true;

      {--------------------------------------------------------------------------}
      { Entonces recorremos el tablero casilla por casilla.                      }
      {--------------------------------------------------------------------------}
      for f := 1 to Nucleo.Juego.MaxFilas do begin
         for c := 1 to Nucleo.Juego.MaxColum do begin
            {--------------------------------------------------------------------}
            { Y vamos probando si se puede mover en dicha casilla, cuando        }
            { encontremos una casilla en la que poder mover, no habr bloqueo.   }
            {--------------------------------------------------------------------}
            if ValidarMovimiento(Nucleo, color, c, f) then begin
               resultado := false;

               Nucleo.Juego.Fichas := aux;

               break;
            end;
         end;

         if resultado = false then break;
      end;

      {--------------------------------------------------------------------------}
      { Y almacenamos el resultado en la variable que le corresponde.            }
      {--------------------------------------------------------------------------}
      if color = TURNO_BLANCAS then begin
         Nucleo.Juego.BlanBloq := resultado;
      end else begin
         Nucleo.Juego.NegrBloq := resultado;
      end;
   end;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Norte.                  }
{********************************************************************************}
function ComprobarLineaNN (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   inc, cene, anterior: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;
   anterior  := color;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia arriba, partiendo de la base de que la ficha    }
   { anterior es una de las nuestras.                                            }
   {-----------------------------------------------------------------------------}
   for inc := (f - 1) downto 1 do begin
      {--------------------------------------------------------------------------}
      { Si la anterior es de nuestro bando podemos encontrarnos varios casos.    }
      {--------------------------------------------------------------------------}
      if anterior = color then begin
         {-----------------------------------------------------------------------}
         { Nos encontramos con una de nuestro propio color, luego no podemos     }
         { trazar una linea en esa direccin.                                    }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[inc, c] = color then begin
            break;

         {-----------------------------------------------------------------------}
         { Es una enemiga, entonces vamos en el buen camino.                     }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[inc, c] = cene then begin
            anterior := cene;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[inc, c] = FICHA_VACIA then begin
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si la anterior es del bando contrario podemos encontrarnos varios casos. }
      {--------------------------------------------------------------------------}
      end else if anterior = cene then begin
         {-----------------------------------------------------------------------}
         { Si encontramos una de nuestro color, entonces podremos trazar la      }
         { linea que estamos buscando.                                           }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[inc, c] = color then begin
            resultado := true;
            break;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[inc, c] = FICHA_VACIA then begin
            break;
         end;
      end;
   end;

   ComprobarLineaNN := resultado;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Sur.                   }
{********************************************************************************}
function ComprobarLineaSS (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   inc, cene, anterior: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;
   anterior  := color;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia abajo, partiendo de la base de que la ficha     }
   { anterior es una de las nuestras.                                            }
   {-----------------------------------------------------------------------------}
   for inc := (f + 1) to Nucleo.Juego.MaxFilas do begin
      {--------------------------------------------------------------------------}
      { Si la anterior es de nuestro bando podemos encontrarnos varios casos.    }
      {--------------------------------------------------------------------------}
      if anterior = color then begin
         {-----------------------------------------------------------------------}
         { Nos encontramos con una de nuestro propio color, luego no podemos     }
         { trazar una linea en esa direccin.                                    }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[inc, c] = color then begin
            break;

         {-----------------------------------------------------------------------}
         { Es una enemiga, entonces vamos en el buen camino.                     }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[inc, c] = cene then begin
            anterior := cene;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[inc, c] = FICHA_VACIA then begin
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si la anterior es del bando contrario podemos encontrarnos varios casos. }
      {--------------------------------------------------------------------------}
      end else if anterior = cene then begin
         {-----------------------------------------------------------------------}
         { Si encontramos una de nuestro color, entonces podremos trazar la      }
         { linea que estamos buscando.                                           }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[inc, c] = color then begin
            resultado := true;
            break;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[inc, c] = FICHA_VACIA then begin
            break;
         end;
      end;
   end;

   ComprobarLineaSS := resultado;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Oeste.                  }
{********************************************************************************}
function ComprobarLineaOO (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   inc, cene, anterior: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;
   anterior  := color;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Recorremos la fila hacia la izquierda, partiendo de la base de que la ficha }
   { anterior es una de las nuestras.                                            }
   {-----------------------------------------------------------------------------}
   for inc := (c - 1) downto 1 do begin
      {--------------------------------------------------------------------------}
      { Si la anterior es de nuestro bando podemos encontrarnos varios casos.    }
      {--------------------------------------------------------------------------}
      if anterior = color then begin
         {-----------------------------------------------------------------------}
         { Nos encontramos con una de nuestro propio color, luego no podemos     }
         { trazar una linea en esa direccin.                                    }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f, inc] = color then begin
            break;

         {-----------------------------------------------------------------------}
         { Es una enemiga, entonces vamos en el buen camino.                     }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f, inc] = cene then begin
            anterior := cene;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f, inc] = FICHA_VACIA then begin
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si la anterior es del bando contrario podemos encontrarnos varios casos. }
      {--------------------------------------------------------------------------}
      end else if anterior = cene then begin
         {-----------------------------------------------------------------------}
         { Si encontramos una de nuestro color, entonces podremos trazar la      }
         { linea que estamos buscando.                                           }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f, inc] = color then begin
            resultado := true;
            break;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f, inc] = FICHA_VACIA then begin
            break;
         end;
      end;
   end;

   ComprobarLineaOO := resultado;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Este.                   }
{********************************************************************************}
function ComprobarLineaEE (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   inc, cene, anterior: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;
   anterior  := color;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Recorremos la fila hacia la derecha, partiendo de la base de que la ficha   }
   { anterior es una de las nuestras.                                            }
   {-----------------------------------------------------------------------------}
   for inc := (c + 1) to Nucleo.Juego.MaxColum do begin
      {--------------------------------------------------------------------------}
      { Si la anterior es de nuestro bando podemos encontrarnos varios casos.    }
      {--------------------------------------------------------------------------}
      if anterior = color then begin
         {-----------------------------------------------------------------------}
         { Nos encontramos con una de nuestro propio color, luego no podemos     }
         { trazar una linea en esa direccin.                                    }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f, inc] = color then begin
            break;

         {-----------------------------------------------------------------------}
         { Es una enemiga, entonces vamos en el buen camino.                     }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f, inc] = cene then begin
            anterior := cene;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f, inc] = FICHA_VACIA then begin
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si la anterior es del bando contrario podemos encontrarnos varios casos. }
      {--------------------------------------------------------------------------}
      end else if anterior = cene then begin
         {-----------------------------------------------------------------------}
         { Si encontramos una de nuestro color, entonces podremos trazar la      }
         { linea que estamos buscando.                                           }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f, inc] = color then begin
            resultado := true;
            break;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f, inc] = FICHA_VACIA then begin
            break;
         end;
      end;
   end;

   ComprobarLineaEE := resultado;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Norte.                  }
{********************************************************************************}
procedure TrazarLineaNN (var Nucleo: tNucleo; color, c, f: byte);
var
   inc, cene: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Ponemos una ficha de nuestro color.                                         }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Fichas[f, c] := color;

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia arriba.                                         }
   {-----------------------------------------------------------------------------}
   for inc := (f - 1) downto 1 do begin
      {--------------------------------------------------------------------------}
      { Cuando nos encontremos con una ficha nuestra, o vaca, paramos.          }
      {--------------------------------------------------------------------------}
      if (Nucleo.Juego.Fichas[inc, c] = color) or
         (Nucleo.Juego.Fichas[inc, c] = FICHA_VACIA) then begin
         break;

      {--------------------------------------------------------------------------}
      { Si es una enemiga, entonces la transformamos.                            }
      {--------------------------------------------------------------------------}
      end else if Nucleo.Juego.Fichas[inc, c] = cene then begin
         Nucleo.Juego.Fichas[inc, c] := color;
      end;
   end;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Sur.                   }
{********************************************************************************}
procedure TrazarLineaSS (var Nucleo: tNucleo; color, c, f: byte);
var
   inc, cene: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Ponemos una ficha de nuestro color.                                         }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Fichas[f, c] := color;

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia abajo.                                          }
   {-----------------------------------------------------------------------------}
   for inc := (f + 1) to Nucleo.Juego.MaxFilas do begin
      {--------------------------------------------------------------------------}
      { Cuando nos encontremos con una ficha nuestra, o vaca, paramos.          }
      {--------------------------------------------------------------------------}
      if (Nucleo.Juego.Fichas[inc, c] = color) or
         (Nucleo.Juego.Fichas[inc, c] = FICHA_VACIA) then begin
         break;

      {--------------------------------------------------------------------------}
      { Si es una enemiga, entonces la transformamos.                            }
      {--------------------------------------------------------------------------}
      end else if Nucleo.Juego.Fichas[inc, c] = cene then begin
         Nucleo.Juego.Fichas[inc, c] := color;
      end;
   end;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Oeste.                  }
{********************************************************************************}
procedure TrazarLineaOO (var Nucleo: tNucleo; color, c, f: byte);
var
   inc, cene: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Ponemos una ficha de nuestro color.                                         }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Fichas[f, c] := color;

   {-----------------------------------------------------------------------------}
   { Recorremos la fila hacia la izquierda.                                      }
   {-----------------------------------------------------------------------------}
   for inc := (c - 1) downto 1 do begin
      {--------------------------------------------------------------------------}
      { Cuando nos encontremos con una ficha nuestra, o vaca, paramos.          }
      {--------------------------------------------------------------------------}
      if (Nucleo.Juego.Fichas[f, inc] = color) or
         (Nucleo.Juego.Fichas[f, inc] = FICHA_VACIA) then begin
         break;

      {--------------------------------------------------------------------------}
      { Si es una enemiga, entonces la transformamos.                            }
      {--------------------------------------------------------------------------}
      end else if Nucleo.Juego.Fichas[f, inc] = cene then begin
         Nucleo.Juego.Fichas[f, inc] := color;
      end;
   end;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Este.                   }
{********************************************************************************}
procedure TrazarLineaEE (var Nucleo: tNucleo; color, c, f: byte);
var
   inc, cene: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Ponemos una ficha de nuestro color.                                         }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Fichas[f, c] := color;

   {-----------------------------------------------------------------------------}
   { Recorremos la fila hacia la derecha.                                        }
   {-----------------------------------------------------------------------------}
   for inc := (c + 1) to Nucleo.Juego.MaxColum do begin
      {--------------------------------------------------------------------------}
      { Cuando nos encontremos con una ficha nuestra, o vaca, paramos.          }
      {--------------------------------------------------------------------------}
      if (Nucleo.Juego.Fichas[f, inc] = color) or
         (Nucleo.Juego.Fichas[f, inc] = FICHA_VACIA) then begin
         break;

      {--------------------------------------------------------------------------}
      { Si es una enemiga, entonces la transformamos.                            }
      {--------------------------------------------------------------------------}
      end else if Nucleo.Juego.Fichas[f, inc] = cene then begin
         Nucleo.Juego.Fichas[f, inc] := color;
      end;
   end;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Norte-Oeste.            }
{********************************************************************************}
function ComprobarLineaNO (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   inc, cene, anterior, menor: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;
   anterior  := color;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Cogemos la medida menor.                                                    }
   {-----------------------------------------------------------------------------}
   if (f - 1) < (c - 1) then
      menor := (f - 1)
   else
      menor := (c - 1);

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia arriba, partiendo de la base de que la ficha    }
   { anterior es una de las nuestras.                                            }
   {-----------------------------------------------------------------------------}
   for inc := 1 to menor do begin
      {--------------------------------------------------------------------------}
      { Si la anterior es de nuestro bando podemos encontrarnos varios casos.    }
      {--------------------------------------------------------------------------}
      if anterior = color then begin
         {-----------------------------------------------------------------------}
         { Nos encontramos con una de nuestro propio color, luego no podemos     }
         { trazar una linea en esa direccin.                                    }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f - inc, c - inc] = color then begin
            break;

         {-----------------------------------------------------------------------}
         { Es una enemiga, entonces vamos en el buen camino.                     }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f - inc, c - inc] = cene then begin
            anterior := cene;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f - inc, c - inc] = FICHA_VACIA then begin
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si la anterior es del bando contrario podemos encontrarnos varios casos. }
      {--------------------------------------------------------------------------}
      end else if anterior = cene then begin
         {-----------------------------------------------------------------------}
         { Si encontramos una de nuestro color, entonces podremos trazar la      }
         { linea que estamos buscando.                                           }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f - inc, c - inc] = color then begin
            resultado := true;
            break;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f - inc, c - inc] = FICHA_VACIA then begin
            break;
         end;
      end;
   end;

   ComprobarLineaNO := resultado;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Norte-Este.             }
{********************************************************************************}
function ComprobarLineaNE (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   inc, cene, anterior, menor: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;
   anterior  := color;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Cogemos la medida menor.                                                    }
   {-----------------------------------------------------------------------------}
   if (f - 1) < (Nucleo.Juego.MaxColum - c) then
      menor := (f - 1)
   else
      menor := (Nucleo.Juego.MaxColum - c);

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia arriba, partiendo de la base de que la ficha    }
   { anterior es una de las nuestras.                                            }
   {-----------------------------------------------------------------------------}
   for inc := 1 to menor do begin
      {--------------------------------------------------------------------------}
      { Si la anterior es de nuestro bando podemos encontrarnos varios casos.    }
      {--------------------------------------------------------------------------}
      if anterior = color then begin
         {-----------------------------------------------------------------------}
         { Nos encontramos con una de nuestro propio color, luego no podemos     }
         { trazar una linea en esa direccin.                                    }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f - inc, c + inc] = color then begin
            break;

         {-----------------------------------------------------------------------}
         { Es una enemiga, entonces vamos en el buen camino.                     }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f - inc, c + inc] = cene then begin
            anterior := cene;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f - inc, c + inc] = FICHA_VACIA then begin
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si la anterior es del bando contrario podemos encontrarnos varios casos. }
      {--------------------------------------------------------------------------}
      end else if anterior = cene then begin
         {-----------------------------------------------------------------------}
         { Si encontramos una de nuestro color, entonces podremos trazar la      }
         { linea que estamos buscando.                                           }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f - inc, c + inc] = color then begin
            resultado := true;
            break;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f - inc, c + inc] = FICHA_VACIA then begin
            break;
         end;
      end;
   end;

   ComprobarLineaNE := resultado;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Norte-Oeste.            }
{********************************************************************************}
function ComprobarLineaSO (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   inc, cene, anterior, menor: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;
   anterior  := color;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Cogemos la medida menor.                                                    }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.MaxFilas - f) < (c - 1) then
      menor := (Nucleo.Juego.MaxFilas - f)
   else
      menor := (c - 1);

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia arriba, partiendo de la base de que la ficha    }
   { anterior es una de las nuestras.                                            }
   {-----------------------------------------------------------------------------}
   for inc := 1 to menor do begin
      {--------------------------------------------------------------------------}
      { Si la anterior es de nuestro bando podemos encontrarnos varios casos.    }
      {--------------------------------------------------------------------------}
      if anterior = color then begin
         {-----------------------------------------------------------------------}
         { Nos encontramos con una de nuestro propio color, luego no podemos     }
         { trazar una linea en esa direccin.                                    }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f + inc, c - inc] = color then begin
            break;

         {-----------------------------------------------------------------------}
         { Es una enemiga, entonces vamos en el buen camino.                     }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f + inc, c - inc] = cene then begin
            anterior := cene;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f + inc, c - inc] = FICHA_VACIA then begin
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si la anterior es del bando contrario podemos encontrarnos varios casos. }
      {--------------------------------------------------------------------------}
      end else if anterior = cene then begin
         {-----------------------------------------------------------------------}
         { Si encontramos una de nuestro color, entonces podremos trazar la      }
         { linea que estamos buscando.                                           }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f + inc, c - inc] = color then begin
            resultado := true;
            break;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f + inc, c - inc] = FICHA_VACIA then begin
            break;
         end;
      end;
   end;

   ComprobarLineaSO := resultado;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Norte-Este.             }
{********************************************************************************}
function ComprobarLineaSE (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   inc, cene, anterior, menor: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;
   anterior  := color;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Cogemos la medida menor.                                                    }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.MaxFilas - f) < (Nucleo.Juego.MaxColum - c) then
      menor := (Nucleo.Juego.MaxFilas - f)
   else
      menor := (Nucleo.Juego.MaxColum - c);

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia arriba, partiendo de la base de que la ficha    }
   { anterior es una de las nuestras.                                            }
   {-----------------------------------------------------------------------------}
   for inc := 1 to menor do begin
      {--------------------------------------------------------------------------}
      { Si la anterior es de nuestro bando podemos encontrarnos varios casos.    }
      {--------------------------------------------------------------------------}
      if anterior = color then begin
         {-----------------------------------------------------------------------}
         { Nos encontramos con una de nuestro propio color, luego no podemos     }
         { trazar una linea en esa direccin.                                    }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f + inc, c + inc] = color then begin
            break;

         {-----------------------------------------------------------------------}
         { Es una enemiga, entonces vamos en el buen camino.                     }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f + inc, c + inc] = cene then begin
            anterior := cene;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f + inc, c + inc] = FICHA_VACIA then begin
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si la anterior es del bando contrario podemos encontrarnos varios casos. }
      {--------------------------------------------------------------------------}
      end else if anterior = cene then begin
         {-----------------------------------------------------------------------}
         { Si encontramos una de nuestro color, entonces podremos trazar la      }
         { linea que estamos buscando.                                           }
         {-----------------------------------------------------------------------}
         if Nucleo.Juego.Fichas[f + inc, c + inc] = color then begin
            resultado := true;
            break;

         {-----------------------------------------------------------------------}
         { Si no hay ninguna ficha tampoco podremos trazar la linea.             }
         {-----------------------------------------------------------------------}
         end else if Nucleo.Juego.Fichas[f + inc, c + inc] = FICHA_VACIA then begin
            break;
         end;
      end;
   end;

   ComprobarLineaSE := resultado;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Norte.                  }
{********************************************************************************}
procedure TrazarLineaNO (var Nucleo: tNucleo; color, c, f: byte);
var
   inc, cene, menor: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Cogemos la medida menor.                                                    }
   {-----------------------------------------------------------------------------}
   if (f - 1) < (c - 1) then
      menor := f - 1
   else
      menor := c - 1;

   {-----------------------------------------------------------------------------}
   { Ponemos una ficha de nuestro color.                                         }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Fichas[f, c] := color;

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia arriba.                                         }
   {-----------------------------------------------------------------------------}
   for inc := 1 to menor do begin
      {--------------------------------------------------------------------------}
      { Cuando nos encontremos con una ficha nuestra, o vaca, paramos.          }
      {--------------------------------------------------------------------------}
      if (Nucleo.Juego.Fichas[f - inc, c - inc] = color) or
         (Nucleo.Juego.Fichas[f - inc, c - inc] = FICHA_VACIA) then begin
         break;

      {--------------------------------------------------------------------------}
      { Si es una enemiga, entonces la transformamos.                            }
      {--------------------------------------------------------------------------}
      end else if Nucleo.Juego.Fichas[f - inc, c - inc] = cene then begin
         Nucleo.Juego.Fichas[f - inc, c - inc] := color;
      end;
   end;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Sur.                   }
{********************************************************************************}
procedure TrazarLineaNE (var Nucleo: tNucleo; color, c, f: byte);
var
   inc, cene, menor: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Cogemos la medida menor.                                                    }
   {-----------------------------------------------------------------------------}
   if (f - 1) < (Nucleo.Juego.MaxColum - c) then
      menor := f - 1
   else
      menor := Nucleo.Juego.MaxColum - c;

   {-----------------------------------------------------------------------------}
   { Ponemos una ficha de nuestro color.                                         }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Fichas[f, c] := color;

   {-----------------------------------------------------------------------------}
   { Recorremos la columna hacia arriba.                                         }
   {-----------------------------------------------------------------------------}
   for inc := 1 to menor do begin
      {--------------------------------------------------------------------------}
      { Cuando nos encontremos con una ficha nuestra, o vaca, paramos.          }
      {--------------------------------------------------------------------------}
      if (Nucleo.Juego.Fichas[f - inc, c + inc] = color) or
         (Nucleo.Juego.Fichas[f - inc, c + inc] = FICHA_VACIA) then begin
         break;

      {--------------------------------------------------------------------------}
      { Si es una enemiga, entonces la transformamos.                            }
      {--------------------------------------------------------------------------}
      end else if Nucleo.Juego.Fichas[f - inc, c + inc] = cene then begin
         Nucleo.Juego.Fichas[f - inc, c + inc] := color;
      end;
   end;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Oeste.                  }
{********************************************************************************}
procedure TrazarLineaSO (var Nucleo: tNucleo; color, c, f: byte);
var
   inc, cene, menor: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Cogemos la medida menor.                                                    }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.MaxFilas - f) < (c - 1) then
      menor := Nucleo.Juego.MaxFilas - f
   else
      menor := c - 1;

   {-----------------------------------------------------------------------------}
   { Ponemos una ficha de nuestro color.                                         }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Fichas[f, c] := color;

   {-----------------------------------------------------------------------------}
   { Recorremos la fila hacia abajo.                                             }
   {-----------------------------------------------------------------------------}
   for inc := 1 to menor do begin
      {--------------------------------------------------------------------------}
      { Cuando nos encontremos con una ficha nuestra, o vaca, paramos.          }
      {--------------------------------------------------------------------------}
      if (Nucleo.Juego.Fichas[f + inc, c - inc] = color) or
         (Nucleo.Juego.Fichas[f + inc, c - inc] = FICHA_VACIA) then begin
         break;

      {--------------------------------------------------------------------------}
      { Si es una enemiga, entonces la transformamos.                            }
      {--------------------------------------------------------------------------}
      end else if Nucleo.Juego.Fichas[f + inc, c - inc] = cene then begin
         Nucleo.Juego.Fichas[f + inc, c - inc] := color;
      end;
   end;
end;

{********************************************************************************}
{ Esta funcin comprueba si se puede trazar una linea al Este.                   }
{********************************************************************************}
procedure TrazarLineaSE (var Nucleo: tNucleo; color, c, f: byte);
var
   inc, cene, menor: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver quien es el bando enemigo.                      }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      cene := FICHA_NEGRA
   else
      cene := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Cogemos la medida menor.                                                    }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.MaxFilas - f) < (Nucleo.Juego.MaxColum - c) then
      menor := Nucleo.Juego.MaxFilas - f
   else
      menor := Nucleo.Juego.MaxColum - c;

   {-----------------------------------------------------------------------------}
   { Ponemos una ficha de nuestro color.                                         }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Fichas[f, c] := color;

   {-----------------------------------------------------------------------------}
   { Recorremos la fila hacia abajo.                                             }
   {-----------------------------------------------------------------------------}
   for inc := 1 to menor do begin
      {--------------------------------------------------------------------------}
      { Cuando nos encontremos con una ficha nuestra, o vaca, paramos.          }
      {--------------------------------------------------------------------------}
      if (Nucleo.Juego.Fichas[f + inc, c + inc] = color) or
         (Nucleo.Juego.Fichas[f + inc, c + inc] = FICHA_VACIA) then begin
         break;

      {--------------------------------------------------------------------------}
      { Si es una enemiga, entonces la transformamos.                            }
      {--------------------------------------------------------------------------}
      end else if Nucleo.Juego.Fichas[f + inc, c + inc] = cene then begin
         Nucleo.Juego.Fichas[f + inc, c + inc] := color;
      end;
   end;
end;

{********************************************************************************}
{ Funcin que valida el movimiento que se esta tratando de hacer.                }
{********************************************************************************}
function ValidarMovimiento (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Vamos comprobando cada direccin de la brjula.                             }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.Fichas[f, c] = FICHA_VACIA then begin
      if ComprobarLineaNN(Nucleo, color, c, f) then begin
         TrazarLineaNN(Nucleo, color, c, f);
         resultado := true;
      end;
      
      if ComprobarLineaSS(Nucleo, color, c, f) then begin
         TrazarLineaSS(Nucleo, color, c, f);
         resultado := true;
      end;
      
      if ComprobarLineaOO(Nucleo, color, c, f) then begin
         TrazarLineaOO(Nucleo, color, c, f);
         resultado := true;
      end;
      
      if ComprobarLineaEE(Nucleo, color, c, f) then begin
         TrazarLineaEE(Nucleo, color, c, f);
         resultado := true;
      end;
      
      if ComprobarLineaNO(Nucleo, color, c, f) then begin
         TrazarLineaNO(Nucleo, color, c, f);
         resultado := true;
      end;
      
      if ComprobarLineaNE(Nucleo, color, c, f) then begin
         TrazarLineaNE(Nucleo, color, c, f);
         resultado := true;
      end;
      
      if ComprobarLineaSO(Nucleo, color, c, f) then begin
         TrazarLineaSO(Nucleo, color, c, f);
         resultado := true;
      end;
      
      if ComprobarLineaSE(Nucleo, color, c, f) then begin
         TrazarLineaSE(Nucleo, color, c, f);
         resultado := true;
      end;
   end;

   ValidarMovimiento := resultado;
end;

{********************************************************************************}
end.
{********************************************************************************}