{********************************************************************************}
{ Othello 1.0 - Juego del Othello bajo ms-dos.                                   }
{ Copyright (C) 2005  Gorka Surez Garca                                        }
{                                                                                }
{ Logica.pas is part of "Othello".                                               }
{                                                                                }
{ "Othello" is free software; you can redistribute it and/or                     }
{ modify it under the terms of the GNU General Public License                    }
{ as published by the Free Software Foundation; either version 2                 }
{ of the License, or (at your option) any later version.                         }
{                                                                                }
{ "Othello" is distributed in the hope that it will be useful,                   }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                 }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  }
{ GNU General Public License for more details.                                   }
{                                                                                }
{ You should have received a copy of the GNU General Public License              }
{ along with "Othello"; if not, write to the Free Software                       }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    }
{********************************************************************************}
unit Logica;
{********************************************************************************}
interface {parte publica}
uses Lesslib, Nucleo, LogMovs, IAJuego, crt;
{********************************************************************************}
{ Funciones para controlar la lgica del juego.                                  }
{********************************************************************************}
procedure EjecutarLogica (var Nucleo: tNucleo);


{********************************************************************************}
implementation {parte privada}
{********************************************************************************}


{********************************************************************************}
{ Funcin que cambia el turno del juego.                                         }
{********************************************************************************}
procedure CambiarTurno (var Nucleo: tNucleo);
var
   numneg, numbla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo a ver que blandos estn bloqueados.                      }
   {-----------------------------------------------------------------------------}
   CalcularBloqueo(Nucleo);

   {-----------------------------------------------------------------------------}
   { Si los dos bandos estn bloqueados, se ha terminado la partida.             }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.BlanBloq and Nucleo.Juego.NegrBloq then begin
      numneg := DarNumeroFichasBando(Nucleo, FICHA_NEGRA);
      numbla := DarNumeroFichasBando(Nucleo, FICHA_BLANCA);

      if numneg = numbla then begin
         Nucleo.Juego.Victoria := JUEGO_TABLAS;
      end else if numneg < numbla then begin
         Nucleo.Juego.Victoria := VICTORIA_J1;
      end else begin
         Nucleo.Juego.Victoria := VICTORIA_J2;
      end;

   {-----------------------------------------------------------------------------}
   { Y ya que no ha terminado la partida, si es el turno de las blancas, lo      }
   { cambiamos al de las negras, que ser el siguiente.                          }
   {-----------------------------------------------------------------------------}
   end else if Nucleo.Juego.Turno = TURNO_BLANCAS then begin
      Nucleo.Juego.Turno := TURNO_NEGRAS;

      if Nucleo.Juego.NegrBloq then begin
         Nucleo.Juego.Mensaje := MSJ_NEGBLOQ;
      end else begin
         Nucleo.Juego.Mensaje := MSJ_NINGUNO;
      end;
   
   {-----------------------------------------------------------------------------}
   { Sino es el turno de las blancas, ser el de las negras, as que tendremos   }
   { que cambiarlo al de las blancas.                                            }
   {-----------------------------------------------------------------------------}
   end else begin
      Nucleo.Juego.Turno := TURNO_BLANCAS;

      if Nucleo.Juego.BlanBloq then begin
         Nucleo.Juego.Mensaje := MSJ_BLABLOQ;
      end else begin
         Nucleo.Juego.Mensaje := MSJ_NINGUNO;
      end;
   end;
end;

{********************************************************************************}
{ Funcin que limpia el tablero donde vamos a jugar sobre una imagen.            }
{********************************************************************************}
procedure LimpiarImagenTablero (var Nucleo: tNucleo);
var
   i, j: integer;
begin
   {-----------------------------------------------------------------------------}
   { Pintamos de negro una regin de la imagen donde est el tablero.            }
   {-----------------------------------------------------------------------------}
   for i := 0 to 199 do
      for j := 0 to 210 do begin
         M13hImagenPonPixel(j, i, Nucleo.Texturas.Tablero, COL_NEGRO);
      end;
end;

{********************************************************************************}
{ Pinta una casilla blanca en la imagen del tablero.                             }
{********************************************************************************}
procedure PintarCasillaBlanca (var Nucleo: tNucleo; x, y: integer);
var
   i, j: integer;
   color: byte;
begin
   {-----------------------------------------------------------------------------}
   { Recorremos la superficie de la textura de la casilla, y vamos cogiendo cada }
   { pixel de esta, y poniendolo en la imagen del tablero, en la posicin que le }
   { corresponda segn unos clculos.                                            }
   {-----------------------------------------------------------------------------}
   for i := 0 to CASTEX_ALTO - 1 do
      for j := 0 to CASTEX_ANCHO - 1 do begin
         color := M13hImagenDarPixel(Nucleo.Texturas.PosCasTex[CASTEX_BLANCA].x + j,
            Nucleo.Texturas.PosCasTex[CASTEX_BLANCA].y + i, Nucleo.Texturas.Fichas);

         M13hImagenPonPixel(Nucleo.Juego.TabPosX + (x - 1) * CASTEX_ANCHO + j,
            Nucleo.Juego.TabPosY + (y - 1) * CASTEX_ALTO + i, Nucleo.Texturas.Tablero,
            color);
      end;
end;

{********************************************************************************}
{ Pinta una casilla azul en la imagen del tablero.                               }
{********************************************************************************}
procedure PintarCasillaAzul (var Nucleo: tNucleo; x, y: integer);
var
   i, j: integer;
   color: byte;
begin
   {-----------------------------------------------------------------------------}
   { Recorremos la superficie de la textura de la casilla, y vamos cogiendo cada }
   { pixel de esta, y poniendolo en la imagen del tablero, en la posicin que le }
   { corresponda segn unos clculos.                                            }
   {-----------------------------------------------------------------------------}
   for i := 0 to CASTEX_ALTO - 1 do
      for j := 0 to CASTEX_ANCHO - 1 do begin
         color := M13hImagenDarPixel(Nucleo.Texturas.PosCasTex[CASTEX_AZUL].x + j,
            Nucleo.Texturas.PosCasTex[CASTEX_AZUL].y + i, Nucleo.Texturas.Fichas);

         M13hImagenPonPixel(Nucleo.Juego.TabPosX + (x - 1) * CASTEX_ANCHO + j,
            Nucleo.Juego.TabPosY + (y - 1) * CASTEX_ALTO + i, Nucleo.Texturas.Tablero,
            color);
      end;
end;

{********************************************************************************}
{ Pinta una casilla azul en la imagen del tablero.                               }
{********************************************************************************}
procedure PintarBordeTablero (var Nucleo: tNucleo);
var
   i, x1, y1, x2, y2: integer;
   color: byte;
begin
   {-----------------------------------------------------------------------------}
   { Pillamos el color correspondiente al borde de la casilla.                   }
   {-----------------------------------------------------------------------------}
   color := M13hImagenDarPixel(Nucleo.Texturas.PosCasTex[CASTEX_BLANCA].x,
      Nucleo.Texturas.PosCasTex[CASTEX_BLANCA].y, Nucleo.Texturas.Fichas);

   {-----------------------------------------------------------------------------}
   { Y calculamos las coordenadas de dos esquinas opuestas del cuadrado que se   }
   { va a pintar sobre la imagen del tablero.                                    }
   {-----------------------------------------------------------------------------}
   x1 := Nucleo.Juego.TabPosX - 1;
   y1 := Nucleo.Juego.TabPosY - 1;
   
   x2 := Nucleo.Juego.TabPosX + CASTEX_ANCHO * Nucleo.Juego.MaxColum;
   y2 := Nucleo.Juego.TabPosY + CASTEX_ALTO * Nucleo.Juego.MaxFilas;

   {-----------------------------------------------------------------------------}
   { Pintamos la linea superior del cuadrado.                                    }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.MaxFilas < 10 then
      for i := x1 to x2 do begin
         M13hImagenPonPixel(i, y1, Nucleo.Texturas.Tablero, color);
      end;
   
   {-----------------------------------------------------------------------------}
   { Pintamos la linea inferior del cuadrado.                                    }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.MaxFilas < 10 then
      for i := x1 to x2 do begin
         M13hImagenPonPixel(i, y2, Nucleo.Texturas.Tablero, color);
      end;
   
   {-----------------------------------------------------------------------------}
   { Pintamos la linea izquierda del cuadrado.                                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.MaxColum < 10 then
      for i := y1 to y2 do begin
         M13hImagenPonPixel(x1, i, Nucleo.Texturas.Tablero, color);
      end;
   
   {-----------------------------------------------------------------------------}
   { Pintamos la linea derecha del cuadrado.                                     }
   {-----------------------------------------------------------------------------}
   for i := y1 to y2 do begin
      M13hImagenPonPixel(x2, i, Nucleo.Texturas.Tablero, color);
   end;
end;

{********************************************************************************}
{ Funcin que pinta el tablero donde vamos a jugar sobre una imagen.             }
{********************************************************************************}
procedure CrearImagenTablero (var Nucleo: tNucleo);
var
   i, j, k: integer;
begin
   {-----------------------------------------------------------------------------}
   { Ya que vamos a pintar el tablero, primero limpiaremos la zona de negro.     }
   {-----------------------------------------------------------------------------}
   LimpiarImagenTablero(Nucleo);

   {-----------------------------------------------------------------------------}
   { Calculamos la posicin inicial del tablero.                                 }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.TabPosX := TAB_CENTROX - trunc(Nucleo.Juego.MaxColum * CASTEX_ANCHO / 2);
   Nucleo.Juego.TabPosY := TAB_CENTROY - trunc(Nucleo.Juego.MaxFilas * CASTEX_ALTO / 2);
   
   {-----------------------------------------------------------------------------}
   { Ahora pintamos las casillas que lo componen. Para que queden bien, en caso  }
   { de que las columnas sean un nmero par, hay que realizar una pequea        }
   { operacin, para evitar tener un tablero a rayas.                            }
   {-----------------------------------------------------------------------------}
   k := CASTEX_BLANCA;

   for i := 1 to Nucleo.Juego.MaxFilas do begin
      for j := 1 to Nucleo.Juego.MaxColum do begin
         if k = CASTEX_BLANCA then begin
            PintarCasillaBlanca(Nucleo, j, i);
            k := CASTEX_AZUL;
         end else begin
            PintarCasillaAzul(Nucleo, j, i);
            k := CASTEX_BLANCA;
         end;
      end;

      if (Nucleo.Juego.MaxColum mod 2) = 0 then begin
         if k = CASTEX_BLANCA then
            k := CASTEX_AZUL
         else
            k := CASTEX_BLANCA;
      end;
   end;

   {-----------------------------------------------------------------------------}
   { Y finalmente pintamos el borde del tablero.                                 }
   {-----------------------------------------------------------------------------}
   PintarBordeTablero(Nucleo);
end;

{********************************************************************************}
{ Funcin que inicializa la creacin de una partida nueva.                       }
{********************************************************************************}
procedure InitPartida (var Nucleo: tNucleo);
begin
   {-----------------------------------------------------------------------------}
   { Ponemos a cero el mximo de filas y columnas para luego elegirlos.          }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.MaxFilas := 0;
   Nucleo.Juego.MaxColum := 0;

   {-----------------------------------------------------------------------------}
   { Y vaciamos los nombres de los jugadores.                                    }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.NombreJ1 := '';
   Nucleo.Juego.NombreJ2 := '';
end;

{********************************************************************************}
{ Funcin que inicializa los datos de una partida nueva creada.                  }
{********************************************************************************}
procedure CrearPartida (var Nucleo: tNucleo);
var
   i, j: integer;
begin
   {-----------------------------------------------------------------------------}
   { Creamos la imagen del tablero.                                              }
   {-----------------------------------------------------------------------------}
   CrearImagenTablero(Nucleo);

   {-----------------------------------------------------------------------------}
   { Indicamos que ninguno de los dos bandos est bloqueado.                     }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.BlanBloq := false;
   Nucleo.Juego.NegrBloq := false;

   {-----------------------------------------------------------------------------}
   { Vaciamos el tablero de fichas.                                              }
   {-----------------------------------------------------------------------------}
   for i := 1 to Nucleo.Juego.MaxFilas do
      for j := 1 to Nucleo.Juego.MaxColum do
   {for i := 1 to MAX_FILAS do
      for j := 1 to MAX_COLUMNAS do}
         Nucleo.Juego.Fichas[i, j] := FICHA_VACIA;

   {-----------------------------------------------------------------------------}
   { Calculamos el centro y ponemos las cuatro fichas iniciales.                 }
   {-----------------------------------------------------------------------------}
   i := trunc(Nucleo.Juego.MaxFilas / 2);
   j := trunc(Nucleo.Juego.MaxColum / 2);
   
   Nucleo.Juego.Fichas[i, j]         := FICHA_BLANCA;
   Nucleo.Juego.Fichas[i + 1, j]     := FICHA_NEGRA;
   Nucleo.Juego.Fichas[i, j + 1]     := FICHA_NEGRA;
   Nucleo.Juego.Fichas[i + 1, j + 1] := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Y finalmente iniciamos las ltimas variables.                               }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Mensaje  := MSJ_NINGUNO;
   Nucleo.Juego.Turno    := TURNO_NEGRAS;
   Nucleo.Juego.Victoria := VICTORIA_NO;

   CambiarTurno(Nucleo);
end;

{********************************************************************************}
{ Funcin que crea una partida para un jugador.                                  }
{********************************************************************************}
procedure CrearPartida1Jug (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_1JUG;

   CrearPartida(Nucleo);
end;

{********************************************************************************}
{ Funcin que crea una partida para dos jugadores.                               }
{********************************************************************************}
procedure CrearPartida2Jug (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_2JUG;

   CrearPartida(Nucleo);
end;

{********************************************************************************}
{ Funcin que inicializa el proceso para crear una partida de un jugador.        }
{********************************************************************************}
procedure InitPartida1Jug (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_ELITAM1;

   InitPartida(Nucleo);
end;

{********************************************************************************}
{ Funcin que inicializa el proceso para crear una partida de dos jugadores.     }
{********************************************************************************}
procedure InitPartida2Jug (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_ELITAM2;

   InitPartida(Nucleo);
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla del men.                                     }
{********************************************************************************}
procedure LogicaMenu (var Nucleo: tNucleo);
var
 tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "1 Jugador" ha sido pulsado.                     }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_1JUGADORES]) then
         InitPartida1Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "2 Jugadores" ha sido pulsado.                   }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_2JUGADORES]) then
         InitPartida2Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Cargar" ha sido pulsado.                        }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_CARGAR]) then
         Nucleo.Juego.Estado := EST_CARGAR1;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Ayuda" ha sido pulsado.                         }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_AYUDA]) then
         Nucleo.Juego.Estado := EST_AYUDA1;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Salir" ha sido pulsado.                         }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_SALIR]) then
         Nucleo.Juego.Estado := EST_SALIR1;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Si es ESC vamos a la pantalla de salir del juego.                        }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_SALIR1;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona que botn del tablero ha sido pulsado.                    }
{********************************************************************************}
function DarBotonTablero (var Nucleo: tNucleo): integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Reiniciar" ha sido pulsado.                        }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_REINICIAR]) then
      DarBotonTablero := BOT_REINICIAR;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Salvar" ha sido pulsado.                           }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_SALVAR]) then
      DarBotonTablero := BOT_SALVAR;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Cargar" ha sido pulsado.                           }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_CARGAR]) then
      DarBotonTablero := BOT_CARGAR;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Ayuda" ha sido pulsado.                            }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_PASAR]) then
      DarBotonTablero := BOT_PASAR;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Salir" ha sido pulsado.                            }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_SALIR]) then
      DarBotonTablero := BOT_SALIR;
end;

{********************************************************************************}
{ Funcin que gestiona que casilla del tablero ha sido pulsada.                  }
{********************************************************************************}
procedure DarCasilla (var Nucleo: tNucleo; var x, y: byte);
var
   i, j: byte;
   reg: tRect;
begin
   x := 0;
   y := 0;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el ratn est encima de cada una de las casillas.            }
   {-----------------------------------------------------------------------------}
   for i := 1 to Nucleo.Juego.MaxFilas do
      for j := 1 to Nucleo.Juego.MaxColum do
       begin
         reg.x := (j - 1) * TAB_SEP_X + Nucleo.Juego.TabPosX;
         reg.y := (i - 1) * TAB_SEP_Y + Nucleo.Juego.TabPosY;
         reg.w := TAB_SEP_X;
         reg.h := TAB_SEP_Y;

         if RatonEstaEn2(Nucleo.Raton, reg) then
          begin
            x := j;
            y := i;
          end;
       end;
end;

{********************************************************************************}
{ Funcin que gestiona el pasar de turno de un jugador.                          }
{********************************************************************************}
procedure PasarTurno (var Nucleo: tNucleo);
begin
   if Nucleo.Juego.Turno = TURNO_BLANCAS then begin
      if Nucleo.Juego.BlanBloq then
         CambiarTurno(Nucleo)
      else
         Nucleo.Juego.Mensaje := MSJ_NOPASAR;
   end else begin
      if Nucleo.Juego.NegrBloq then
         CambiarTurno(Nucleo)
      else
         Nucleo.Juego.Mensaje := MSJ_NOPASAR;
   end;
end;

{********************************************************************************}
{ Funcin que gestiona una partida de un solo jugador.                           }
{********************************************************************************}
procedure Logica1Jug (var Nucleo: tNucleo);
var
   tecla: integer;
   c, f, ficha: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo al turno actual.                                         }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.Turno = TURNO_BLANCAS then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn izquierdo del ratn est pulsado.                }
      {--------------------------------------------------------------------------}
      if Nucleo.Raton.bi then
       begin
         {-----------------------------------------------------------------------}
         { Comprobamos primero si ha pulsado alguno de los botones.              }
         {-----------------------------------------------------------------------}
         case DarBotonTablero(Nucleo) of
            BOT_REINICIAR: Nucleo.Juego.Estado := EST_REINICIAR1;
            BOT_SALVAR:    Nucleo.Juego.Estado := EST_SALVAR1;
            BOT_CARGAR:    Nucleo.Juego.Estado := EST_CARGAR2;
            BOT_PASAR:     PasarTurno(Nucleo);
            BOT_SALIR:     Nucleo.Juego.Estado := EST_SALIR2;
         end;

         {-----------------------------------------------------------------------}
         { Obtenemos la casilla que hay pulsada actualmente.                     }
         {-----------------------------------------------------------------------}
         DarCasilla(Nucleo, c, f);

         {-----------------------------------------------------------------------}
         { Comprobamos que hemos recibido unas coordenadas validas.              }
         {-----------------------------------------------------------------------}
         if (0 < c) and (c <= Nucleo.Juego.MaxColum) and
            (0 < f) and (f <= Nucleo.Juego.MaxFilas) and
            (Nucleo.Juego.Fichas[f, c] = FICHA_VACIA) then begin

            {--------------------------------------------------------------------}
            { Dado que es una casilla vaca, validamos el movimiento.            }
            {--------------------------------------------------------------------}
            if ValidarMovimiento(Nucleo, Nucleo.Juego.Turno, c, f) then begin
               Nucleo.Juego.Mensaje := MSJ_NINGUNO;

               CambiarTurno(Nucleo);

            {--------------------------------------------------------------------}
            { Si es un movimiento erroneo, lanzamos un mensaje.                  }
            {--------------------------------------------------------------------}
            end else begin
               Nucleo.Juego.Mensaje := MSJ_MOVINV;
            end;
         end;

         {-----------------------------------------------------------------------}
         { Retardo para evitar saltos indeseados.                                }
         {-----------------------------------------------------------------------}
         DoDelay(TIEMPO);
       end
      else
       begin
         {-----------------------------------------------------------------------}
         { Comprobamos el valor de la ltima tecla pulsada.                      }
         {-----------------------------------------------------------------------}
         tecla := DarTecla;

         {-----------------------------------------------------------------------}
         { Si es ESC vamos a la pantalla de salir de la partida.                 }
         {-----------------------------------------------------------------------}
         if tecla = TECLA_ESC then
            Nucleo.Juego.Estado := EST_SALIR2;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Si es el de las negras mueve la mquina.                                    }
   {-----------------------------------------------------------------------------}
   else
    begin
      if Nucleo.Juego.NegrBloq then begin
         PasarTurno(Nucleo);
      end else begin
         MaquinaMueve(Nucleo);
         CambiarTurno(Nucleo);
      end;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona el cuadro de dialogo de victoria en un solo jugador.      }
{********************************************************************************}
procedure DlgVictoria1Jug (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         InitPartida1Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona una partida de un solo jugador.                           }
{********************************************************************************}
procedure Partida1Jug (var Nucleo: tNucleo);
begin
   case Nucleo.Juego.Victoria of
      VICTORIA_NO:  Logica1Jug(Nucleo);
      VICTORIA_J1:  DlgVictoria1Jug(Nucleo);
      VICTORIA_J2:  DlgVictoria1Jug(Nucleo);
      JUEGO_TABLAS: DlgVictoria1Jug(Nucleo);
   end;
end;

{********************************************************************************}
{ Funcin que gestiona una partida de dos jugadores.                             }
{********************************************************************************}
procedure Logica2Jug (var Nucleo: tNucleo);
var
   tecla: integer;
   c, f, ficha: byte;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos primero si ha pulsado alguno de los botones.                 }
      {--------------------------------------------------------------------------}
      case DarBotonTablero(Nucleo) of
         BOT_REINICIAR: Nucleo.Juego.Estado := EST_REINICIAR2;
         BOT_SALVAR:    Nucleo.Juego.Estado := EST_SALVAR2;
         BOT_CARGAR:    Nucleo.Juego.Estado := EST_CARGAR3;
         BOT_PASAR:     PasarTurno(Nucleo);
         BOT_SALIR:     Nucleo.Juego.Estado := EST_SALIR3;
      end;

      {--------------------------------------------------------------------------}
      { Obtenemos la casilla que hay pulsada actualmente.                        }
      {--------------------------------------------------------------------------}
      DarCasilla(Nucleo, c, f);

      {--------------------------------------------------------------------------}
      { Comprobamos que hemos recibido unas coordenadas validas.                 }
      {--------------------------------------------------------------------------}
      if (0 < c) and (c <= Nucleo.Juego.MaxColum) and
         (0 < f) and (f <= Nucleo.Juego.MaxFilas) and
         (Nucleo.Juego.Fichas[f, c] = FICHA_VACIA) then begin
         {-----------------------------------------------------------------------}
         { Dado que es una casilla vaca, validamos el movimiento.               }
         {-----------------------------------------------------------------------}
         if ValidarMovimiento(Nucleo, Nucleo.Juego.Turno, c, f) then begin
            Nucleo.Juego.Mensaje := MSJ_NINGUNO;

            CambiarTurno(Nucleo);

         {-----------------------------------------------------------------------}
         { Si es un movimiento erroneo, lanzamos un mensaje.                     }
         {-----------------------------------------------------------------------}
         end else begin
            Nucleo.Juego.Mensaje := MSJ_MOVINV;
         end;
      end;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Si es ESC vamos a la pantalla de salir de la partida.                    }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_SALIR3;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona el cuadro de dialogo de victoria en un solo jugador.      }
{********************************************************************************}
procedure DlgVictoria2Jug (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         InitPartida2Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona una partida de un solo jugador.                           }
{********************************************************************************}
procedure Partida2Jug (var Nucleo: tNucleo);
begin
   case Nucleo.Juego.Victoria of
      VICTORIA_NO:  Logica2Jug(Nucleo);
      VICTORIA_J1:  DlgVictoria2Jug(Nucleo);
      VICTORIA_J2:  DlgVictoria2Jug(Nucleo);
      JUEGO_TABLAS: DlgVictoria2Jug(Nucleo);
   end;
end;

{********************************************************************************}
{ Funcin que da el nmero de los nombres de las partidas guardadas pinchado.    }
{********************************************************************************}
procedure DarPartidaSeleccionada (var Nucleo: tNucleo);
var
   i: byte;
   r: tRect;
begin
   Nucleo.Partidas.Parsel := 0;

   {-----------------------------------------------------------------------------}
   { Vamos uno por uno comprobando cual ha sido pinchado.                        }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_PARTIDAS do
    begin
      r.x := PART_INITX;
      r.y := PART_INITY + PART_SEP_Y * (i - 1);
      r.w := PART_INITW;
      r.h := PART_INITH;

      if RatonEstaEn2(Nucleo.Raton, r) then
         Nucleo.Partidas.Parsel := i;
    end;
end;

{********************************************************************************}
{ Funcin que salva una partida.                                                 }
{********************************************************************************}
procedure SalvarPartida (var Nucleo: tNucleo);
var
   f: tFichPartida;
   reg: tRegPartida;
begin
   {-----------------------------------------------------------------------------}
   { Asignamos el fichero a la variable f.                                       }
   {-----------------------------------------------------------------------------}
   assign(f, Nucleo.Partidas.Fichero[Nucleo.Partidas.Parsel]);

   {-----------------------------------------------------------------------------}
   { Lo abrimos para escribir.                                                   }
   {-----------------------------------------------------------------------------}
   {$I-}
   rewrite(f);
   {$I+}
   
   {-----------------------------------------------------------------------------}
   { Si no hay ningn fallo escribimos su contenido.                             }
   {-----------------------------------------------------------------------------}
   if IOResult = 0 then
    begin
      reg.Nombre := Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel];
      reg.Juego := Nucleo.Juego;

      if Nucleo.Juego.Estado = EST_SALVAR1 then
         reg.Juego.Estado := EST_1JUG
      else if Nucleo.Juego.Estado = EST_SALVAR2 then
         reg.Juego.Estado := EST_2JUG;

      write(f, reg);      

      close(f);
    end;
end;

{********************************************************************************}
{ Funcin que carga una partida.                                                 }
{********************************************************************************}
procedure CargarPartida (var Nucleo: tNucleo);
var
   f: tFichPartida;
   reg: tRegPartida;
begin
   {-----------------------------------------------------------------------------}
   { Asignamos el fichero a la variable f.                                       }
   {-----------------------------------------------------------------------------}
   assign(f, Nucleo.Partidas.Fichero[Nucleo.Partidas.Parsel]);

   {-----------------------------------------------------------------------------}
   { Lo abrimos para leer.                                                       }
   {-----------------------------------------------------------------------------}
   {$I-}
   reset(f);
   {$I+}
   
   {-----------------------------------------------------------------------------}
   { Si no hay ningn fallo leemos su contenido.                                 }
   {-----------------------------------------------------------------------------}
   if IOResult = 0 then
    begin
      if filesize(f) > 0 then
       begin
         read(f, reg);
         Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel] := reg.Nombre;
         reg.Juego.Estado := Nucleo.Juego.Estado;
         Nucleo.Juego := reg.Juego;
       end;

      close(f);
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de salvar una partida.                        }
{********************************************************************************}
procedure LogicaSalvar (var Nucleo: tNucleo);
var
   tecla, tam, i: integer;
   auxcad: string;
begin
   case Nucleo.Partidas.Estado of
      {--------------------------------------------------------------------------}
      { Pantalla normal de presentacin de salvar partidas.                      }
      {--------------------------------------------------------------------------}
      PART_EST_NORMAL:
       begin
         if Nucleo.Raton.bi then
          begin
            {--------------------------------------------------------------------}
            { Buscamos la partida seleccionada.                                  }
            {--------------------------------------------------------------------}
            DarPartidaSeleccionada(Nucleo);

            if Nucleo.Partidas.Parsel <> 0 then
             begin
               if Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel] = '---' then
                begin
                  Nucleo.Partidas.Auxcad := '';
                  Nucleo.Partidas.Estado := PART_EST_METER
                end
               else
                begin
                  Nucleo.Partidas.Auxcad := Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel];
                  Nucleo.Partidas.Estado := PART_EST_AVISO;
                end;
             end;

            {--------------------------------------------------------------------}
            { Retardo para evitar saltos indeseados.                             }
            {--------------------------------------------------------------------}
            DoDelay(TIEMPO);
          end
         else
          begin
            tecla := DarTecla;

            {--------------------------------------------------------------------}
            { Si pulsa escape salimos de la pantalla.                            }
            {--------------------------------------------------------------------}
            if tecla = TECLA_ESC then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

               if Nucleo.Juego.Estado = EST_SALVAR1 then
                  Nucleo.Juego.Estado := EST_1JUG
               else if Nucleo.Juego.Estado = EST_SALVAR2 then
                  Nucleo.Juego.Estado := EST_2JUG;
             end;
          end;
       end;

      {--------------------------------------------------------------------------}
      { Aviso sobre si desea sobre-escribir una partida.                         }
      {--------------------------------------------------------------------------}
      PART_EST_AVISO:
       begin
         if Nucleo.Raton.bi then
          begin
            {--------------------------------------------------------------------}
            { Comprobamos si el botn "S" ha sido pulsado.                      }
            {--------------------------------------------------------------------}
            if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
               Nucleo.Partidas.Estado := PART_EST_METER;

            {--------------------------------------------------------------------}
            { Comprobamos si el botn "No" ha sido pulsado.                      }
            {--------------------------------------------------------------------}
            if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

            {--------------------------------------------------------------------}
            { Retardo para evitar saltos indeseados.                             }
            {--------------------------------------------------------------------}
            DoDelay(TIEMPO);
          end
         else
          begin
            tecla := DarTecla;

            {--------------------------------------------------------------------}
            { Si pulsa escape salimos del dilogo.                               }
            {--------------------------------------------------------------------}
            if tecla = TECLA_ESC then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;
             end;
          end;
       end;

      {--------------------------------------------------------------------------}
      { Seccin para meter el nombre de la partida.                              }
      {--------------------------------------------------------------------------}
      PART_EST_METER:
       begin
         tecla := DarTecla;
         
         {-----------------------------------------------------------------------}
         { Si pulsa escape salimos de la pantalla.                               }
         {-----------------------------------------------------------------------}
         if tecla = TECLA_ESC then
          begin
            Nucleo.Partidas.Estado := PART_EST_NORMAL;

            if Nucleo.Juego.Estado = EST_SALVAR1 then
               Nucleo.Juego.Estado := EST_1JUG
            else if Nucleo.Juego.Estado = EST_SALVAR2 then
               Nucleo.Juego.Estado := EST_2JUG
          end
         {-----------------------------------------------------------------------}
         { Si pulsa enter salvamos la partida.                                   }
         {-----------------------------------------------------------------------}
         else if (tecla = TECLA_ENTER) and (Nucleo.Partidas.Auxcad <> '') then
          begin
            Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel] := Nucleo.Partidas.Auxcad;

            SalvarPartida(Nucleo);

            Nucleo.Partidas.Estado := PART_EST_NORMAL;

            if Nucleo.Juego.Estado = EST_SALVAR1 then
               Nucleo.Juego.Estado := EST_1JUG
            else if Nucleo.Juego.Estado = EST_SALVAR2 then
               Nucleo.Juego.Estado := EST_2JUG;
          end
         {-----------------------------------------------------------------------}
         { Si pulsamos retroceso borramos un caracter.                           }
         {-----------------------------------------------------------------------}
         else if (tecla = TECLA_RETROCESO) and (Nucleo.Partidas.Auxcad <> '') then
          begin
            tam := length(Nucleo.Partidas.Auxcad) - 1;
            auxcad := Nucleo.Partidas.Auxcad;
            Nucleo.Partidas.Auxcad := '';

            for i := 1 to tam do
               Nucleo.Partidas.Auxcad := Nucleo.Partidas.Auxcad + auxcad[i]; 
          end
         {-----------------------------------------------------------------------}
         { Y sino metemos el caracter que sea en la cadena.                      }
         {-----------------------------------------------------------------------}
         else if (tecla <> 0) and (tecla <> TECLA_ENTER) and (tecla <> TECLA_RETROCESO) then
          begin
            Nucleo.Partidas.Auxcad := Nucleo.Partidas.Auxcad + chr(tecla);
            DoDelay(TIEMPO);
          end;
       end;
   end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de cargar una partida contra la maquina.      }
{********************************************************************************}
procedure LogicaCargar (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   case Nucleo.Partidas.Estado of
      {--------------------------------------------------------------------------}
      { Pantalla normal de presentacin de salvar partidas.                      }
      {--------------------------------------------------------------------------}
      PART_EST_NORMAL:
       begin
         if Nucleo.Raton.bi then
          begin
            {--------------------------------------------------------------------}
            { Buscamos la partida seleccionada.                                  }
            {--------------------------------------------------------------------}
            DarPartidaSeleccionada(Nucleo);

            if Nucleo.Partidas.Parsel <> 0 then
               if Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel] <> '---' then
                  Nucleo.Partidas.Estado := PART_EST_AVISO;

            {--------------------------------------------------------------------}
            { Retardo para evitar saltos indeseados.                             }
            {--------------------------------------------------------------------}
            DoDelay(TIEMPO);
          end
         else
          begin
            tecla := DarTecla;

            {--------------------------------------------------------------------}
            { Si pulsa escape salimos de la pantalla.                            }
            {--------------------------------------------------------------------}
            if tecla = TECLA_ESC then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

               if Nucleo.Juego.Estado = EST_CARGAR1 then
                  Nucleo.Juego.Estado := EST_MENU
               else if Nucleo.Juego.Estado = EST_CARGAR2 then
                  Nucleo.Juego.Estado := EST_1JUG
               else if Nucleo.Juego.Estado = EST_CARGAR3 then
                  Nucleo.Juego.Estado := EST_2JUG;
             end;
          end;
       end;

      {--------------------------------------------------------------------------}
      { Aviso sobre si desea cargar una partida.                                 }
      {--------------------------------------------------------------------------}
      PART_EST_AVISO:
       begin
         if Nucleo.Raton.bi then
          begin
            {--------------------------------------------------------------------}
            { Comprobamos si el botn "S" ha sido pulsado.                      }
            {--------------------------------------------------------------------}
            if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

               CargarPartida(Nucleo);
               
               if Nucleo.Juego.Estado = EST_CARGAR1 then
                  Nucleo.Juego.Estado := EST_1JUG
               else if Nucleo.Juego.Estado = EST_CARGAR2 then
                  Nucleo.Juego.Estado := EST_1JUG
               else if Nucleo.Juego.Estado = EST_CARGAR3 then
                  Nucleo.Juego.Estado := EST_2JUG;
             end;

            {--------------------------------------------------------------------}
            { Comprobamos si el botn "No" ha sido pulsado.                      }
            {--------------------------------------------------------------------}
            if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

            {--------------------------------------------------------------------}
            { Retardo para evitar saltos indeseados.                             }
            {--------------------------------------------------------------------}
            DoDelay(TIEMPO);
          end
         else
          begin
            tecla := DarTecla;

            {--------------------------------------------------------------------}
            { Si pulsa escape salimos del dilogo.                               }
            {--------------------------------------------------------------------}
            if tecla = TECLA_ESC then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;
             end;
          end;
       end;
   end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de ayuda.                                     }
{********************************************************************************}
procedure LogicaAyuda1 (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_MENU;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de salir del juego.                           }
{********************************************************************************}
procedure LogicaSalir1 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         Nucleo.Juego.Estado := EST_FIN;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de salir de la partida contra la maquina.     }
{********************************************************************************}
procedure LogicaSalir2 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_1JUG;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_1JUG;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de salir de la partida contra otro jugador.   }
{********************************************************************************}
procedure LogicaSalir3 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_2JUG;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_2JUG;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de reiniciar la partida contra la maquina.    }
{********************************************************************************}
procedure LogicaReiniciar1 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         CrearPartida1Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_1JUG;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_1JUG;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de reiniciar la partida contra otro jugador.  }
{********************************************************************************}
procedure LogicaReiniciar2 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         CrearPartida2Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_2JUG;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_2JUG;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de elegir tamao para un jugador.             }
{********************************************************************************}
procedure LogicaElegirTam1 (var Nucleo: tNucleo);
var
   tecla: integer;
   i: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver si hemos pulsado alguno de los botones.         }
   {-----------------------------------------------------------------------------}
   for i := TAMBOT_COLINI to TAMBOT_COLFIN do begin
      if RatonEstaEn(Nucleo.Raton, TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X,
         TAMBOT_INI1Y, TAMBOT_ANCHO, TAMBOT_ALTO) and Nucleo.Raton.bi then begin
         Nucleo.Juego.MaxColum := i;
      end;
   end;
   
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver si hemos pulsado alguno de los botones.         }
   {-----------------------------------------------------------------------------}
   for i := TAMBOT_COLINI to TAMBOT_COLFIN do begin
      if RatonEstaEn(Nucleo.Raton, TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X,
         TAMBOT_INI2Y, TAMBOT_ANCHO, TAMBOT_ALTO) and Nucleo.Raton.bi then begin
         Nucleo.Juego.MaxFilas := i;
      end;
   end;

   {-----------------------------------------------------------------------------}
   { Si hemos elegido ambas dimensiones, pasamos a elegir el nombre del jugador. }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.MaxColum >= TAMBOT_COLINI) and
      (Nucleo.Juego.MaxFilas >= TAMBOT_COLINI) then begin
      Nucleo.Juego.Estado := EST_ELINOM1J1;

   {-----------------------------------------------------------------------------}
   { Si pulsamos escape saltamos al men principal.                              }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Si es ESC vamos a la pantalla de salir del juego.                        }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de elegir el nombre del jugador.              }
{********************************************************************************}
procedure LogicaElegirNombreJug (var Nucleo: tNucleo);
var
   tecla, tam, i: integer;
   auxcad: string;
begin
   {-----------------------------------------------------------------------------}
   { Capturamos la tecla pulsada.                                                }
   {-----------------------------------------------------------------------------}
   tecla := DarTecla;
   
   {-----------------------------------------------------------------------------}
   { Si pulsa escape salimos al men.                                            }
   {-----------------------------------------------------------------------------}
   if tecla = TECLA_ESC then
    begin
      Nucleo.Juego.Estado := EST_MENU;
    end
   {-----------------------------------------------------------------------------}
   { Si pulsa enter elegimos la dificultad.                                      }
   {-----------------------------------------------------------------------------}
   else if (tecla = TECLA_ENTER) and (Nucleo.Juego.NombreJ1 <> '') then
    begin
      Nucleo.Juego.Estado := EST_ELIDIFICUL;
    end
   {-----------------------------------------------------------------------------}
   { Si pulsamos retroceso borramos un caracter.                                 }
   {-----------------------------------------------------------------------------}
   else if (tecla = TECLA_RETROCESO) and (Nucleo.Juego.NombreJ1 <> '') then
    begin
      tam := length(Nucleo.Juego.NombreJ1) - 1;
      auxcad := Nucleo.Juego.NombreJ1;
      Nucleo.Juego.NombreJ1 := '';

      for i := 1 to tam do
         Nucleo.Juego.NombreJ1 := Nucleo.Juego.NombreJ1 + auxcad[i]; 
    end
   {-----------------------------------------------------------------------------}
   { Y sino metemos el caracter que sea en la cadena.                            }
   {-----------------------------------------------------------------------------}
   else if (tecla <> 0) and (tecla <> TECLA_ENTER) and (tecla <> TECLA_RETROCESO) then
    begin
      Nucleo.Juego.NombreJ1 := Nucleo.Juego.NombreJ1 + chr(tecla);
      DoDelay(TIEMPO);
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de elegir la dificultad.                      }
{********************************************************************************}
procedure LogicaElegirDificultad (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Cargar" ha sido pulsado.                        }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.PosDifBot[BOT_DIF_FACIL]) then
       begin
         Nucleo.Juego.Dificultad := DIFICULTAD_FACIL;
         CrearPartida1Jug(Nucleo);
       end;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Ayuda" ha sido pulsado.                         }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.PosDifBot[BOT_DIF_NORMAL]) then
       begin
         Nucleo.Juego.Dificultad := DIFICULTAD_NORMAL;
         CrearPartida1Jug(Nucleo);
       end;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Salir" ha sido pulsado.                         }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.PosDifBot[BOT_DIF_DIFICIL]) then
       begin
         Nucleo.Juego.Dificultad := DIFICULTAD_DIFICIL;
         CrearPartida1Jug(Nucleo);
       end;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Si es ESC vamos a la pantalla de salir del juego.                        }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de elegir tamao para dos jugadores.          }
{********************************************************************************}
procedure LogicaElegirTam2 (var Nucleo: tNucleo);
var
   tecla: integer;
   i: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver si hemos pulsado alguno de los botones.         }
   {-----------------------------------------------------------------------------}
   for i := TAMBOT_COLINI to TAMBOT_COLFIN do begin
      if RatonEstaEn(Nucleo.Raton, TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X,
         TAMBOT_INI1Y, TAMBOT_ANCHO, TAMBOT_ALTO) and Nucleo.Raton.bi then begin
         Nucleo.Juego.MaxColum := i;
      end;
   end;
   
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo para ver si hemos pulsado alguno de los botones.         }
   {-----------------------------------------------------------------------------}
   for i := TAMBOT_COLINI to TAMBOT_COLFIN do begin
      if RatonEstaEn(Nucleo.Raton, TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X,
         TAMBOT_INI2Y, TAMBOT_ANCHO, TAMBOT_ALTO) and Nucleo.Raton.bi then begin
         Nucleo.Juego.MaxFilas := i;
      end;
   end;

   {-----------------------------------------------------------------------------}
   { Si hemos elegido ambas dimensiones, pasamos a elegir el nombre del jugador. }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.MaxColum >= TAMBOT_COLINI) and
      (Nucleo.Juego.MaxFilas >= TAMBOT_COLINI) then begin
      Nucleo.Juego.Estado := EST_ELINOM2J1;

   {-----------------------------------------------------------------------------}
   { Si pulsamos escape saltamos al men principal.                              }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Si es ESC vamos a la pantalla de salir del juego.                        }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end; 
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de elegir el nombre del jugador 1.            }
{********************************************************************************}
procedure LogicaElegirNombreJug1 (var Nucleo: tNucleo);
var
   tecla, tam, i: integer;
   auxcad: string;
begin
   {-----------------------------------------------------------------------------}
   { Capturamos la tecla pulsada.                                                }
   {-----------------------------------------------------------------------------}
   tecla := DarTecla;
   
   {-----------------------------------------------------------------------------}
   { Si pulsa escape salimos al men.                                            }
   {-----------------------------------------------------------------------------}
   if tecla = TECLA_ESC then
    begin
      Nucleo.Juego.Estado := EST_MENU;
    end
   {-----------------------------------------------------------------------------}
   { Si pulsa enter elegimos el siguiente nombre.                                }
   {-----------------------------------------------------------------------------}
   else if (tecla = TECLA_ENTER) and (Nucleo.Juego.NombreJ1 <> '') then
    begin
      Nucleo.Juego.Estado := EST_ELINOM2J2;
    end
   {-----------------------------------------------------------------------------}
   { Si pulsamos retroceso borramos un caracter.                                 }
   {-----------------------------------------------------------------------------}
   else if (tecla = TECLA_RETROCESO) and (Nucleo.Juego.NombreJ1 <> '') then
    begin
      tam := length(Nucleo.Juego.NombreJ1) - 1;
      auxcad := Nucleo.Juego.NombreJ1;
      Nucleo.Juego.NombreJ1 := '';

      for i := 1 to tam do
         Nucleo.Juego.NombreJ1 := Nucleo.Juego.NombreJ1 + auxcad[i]; 
    end
   {-----------------------------------------------------------------------------}
   { Y sino metemos el caracter que sea en la cadena.                            }
   {-----------------------------------------------------------------------------}
   else if (tecla <> 0) and (tecla <> TECLA_ENTER) and (tecla <> TECLA_RETROCESO) then
    begin
      Nucleo.Juego.NombreJ1 := Nucleo.Juego.NombreJ1 + chr(tecla);
      DoDelay(TIEMPO);
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de elegir el nombre del jugador 2.            }
{********************************************************************************}
procedure LogicaElegirNombreJug2 (var Nucleo: tNucleo);
var
   tecla, tam, i: integer;
   auxcad: string;
begin
   {-----------------------------------------------------------------------------}
   { Capturamos la tecla pulsada.                                                }
   {-----------------------------------------------------------------------------}
   tecla := DarTecla;
   
   {-----------------------------------------------------------------------------}
   { Si pulsa escape salimos al men.                                            }
   {-----------------------------------------------------------------------------}
   if tecla = TECLA_ESC then
    begin
      Nucleo.Juego.Estado := EST_MENU;
    end
   {-----------------------------------------------------------------------------}
   { Si pulsa enter creamos ya la partida.                                       }
   {-----------------------------------------------------------------------------}
   else if (tecla = TECLA_ENTER) and (Nucleo.Juego.NombreJ2 <> '') then
    begin
      CrearPartida2Jug(Nucleo);
    end
   {-----------------------------------------------------------------------------}
   { Si pulsamos retroceso borramos un caracter.                                 }
   {-----------------------------------------------------------------------------}
   else if (tecla = TECLA_RETROCESO) and (Nucleo.Juego.NombreJ2 <> '') then
    begin
      tam := length(Nucleo.Juego.NombreJ2) - 1;
      auxcad := Nucleo.Juego.NombreJ2;
      Nucleo.Juego.NombreJ2 := '';

      for i := 1 to tam do
         Nucleo.Juego.NombreJ2 := Nucleo.Juego.NombreJ2 + auxcad[i]; 
    end
   {-----------------------------------------------------------------------------}
   { Y sino metemos el caracter que sea en la cadena.                            }
   {-----------------------------------------------------------------------------}
   else if (tecla <> 0) and (tecla <> TECLA_ENTER) and (tecla <> TECLA_RETROCESO) then
    begin
      Nucleo.Juego.NombreJ2 := Nucleo.Juego.NombreJ2 + chr(tecla);
      DoDelay(TIEMPO);
    end;
end;

{********************************************************************************}
{ Funcin principal que gestiona toda la lgica interna del juego.               }
{********************************************************************************}
procedure EjecutarLogica (var Nucleo: tNucleo);
begin
   RatonEstado(Nucleo.Raton);

   case Nucleo.Juego.Estado of
      EST_MENU:       LogicaMenu(Nucleo);
      EST_1JUG:       Partida1Jug(Nucleo);
      EST_2JUG:       Partida2Jug(Nucleo);
      EST_SALVAR1:    LogicaSalvar(Nucleo);
      EST_SALVAR2:    LogicaSalvar(Nucleo);
      EST_CARGAR1:    LogicaCargar(Nucleo);
      EST_CARGAR2:    LogicaCargar(Nucleo);
      EST_CARGAR3:    LogicaCargar(Nucleo);
      EST_AYUDA1:     LogicaAyuda1(Nucleo);
      EST_SALIR1:     LogicaSalir1(Nucleo);
      EST_SALIR2:     LogicaSalir2(Nucleo);
      EST_SALIR3:     LogicaSalir3(Nucleo);
      EST_REINICIAR1: LogicaReiniciar1(Nucleo);
      EST_REINICIAR2: LogicaReiniciar2(Nucleo);
      EST_ELITAM1:    LogicaElegirTam1(Nucleo);
      EST_ELINOM1J1:  LogicaElegirNombreJug(Nucleo);
      EST_ELIDIFICUL: LogicaElegirDificultad(Nucleo);
      EST_ELITAM2:    LogicaElegirTam2(Nucleo);
      EST_ELINOM2J1:  LogicaElegirNombreJug1(Nucleo);
      EST_ELINOM2J2:  LogicaElegirNombreJug2(Nucleo);
   end;
end;

{********************************************************************************}
end.
{********************************************************************************}