{********************************************************************************}
{ Othello 1.0 - Juego del Othello bajo ms-dos.                                   }
{ Copyright (C) 2005  Gorka Surez Garca                                        }
{                                                                                }
{ Pintado.pas is part of "Othello".                                              }
{                                                                                }
{ "Othello" is free software; you can redistribute it and/or                     }
{ modify it under the terms of the GNU General Public License                    }
{ as published by the Free Software Foundation; either version 2                 }
{ of the License, or (at your option) any later version.                         }
{                                                                                }
{ "Othello" is distributed in the hope that it will be useful,                   }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                 }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  }
{ GNU General Public License for more details.                                   }
{                                                                                }
{ You should have received a copy of the GNU General Public License              }
{ along with "Othello"; if not, write to the Free Software                       }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    }
{********************************************************************************}
unit Pintado;
{********************************************************************************}
interface {parte publica}
uses Lesslib, Nucleo, crt;
{********************************************************************************}
{ Funciones para pintar la pantalla.                                             }
{********************************************************************************}
procedure PintarPantalla (var Nucleo: tNucleo);


{********************************************************************************}
implementation {parte privada}
{********************************************************************************}


{********************************************************************************}
{ Funcin que pinta una linea horizontal.                                        }
{********************************************************************************}
procedure PintarLineaH (xi, xf, y: word; color: byte);
var
   i: word;
begin
   for i := xi to xf do
      M13hPixelPoner(i, y, color);
end;

{********************************************************************************}
{ Funcin que pinta una linea vertical.                                          }
{********************************************************************************}
procedure PintarLineaV (x, yi, yf: word; color: byte);
var
   i: word;
begin
   for i := yi to yf do
      M13hPixelPoner(x, i, color);
end;

{********************************************************************************}
{ Funcin que pinta un rectangulo.                                               }
{********************************************************************************}
procedure PintarRectangulo (xi, xf, yi, yf: word; color: byte);
begin
   PintarLineaH(xi, xf, yi, color);
   PintarLineaV(xi, yi, yf, color);
   PintarLineaV(xf, yi, yf, color);
   PintarLineaH(xi, xf, yf, color);
end;

{********************************************************************************}
{ Funcin que pinta un cuadro de dilogo, indicndole una cadena con el mensaje, }
{ y la posicin x en la que se va a centrar dicho mensaje.                       }
{********************************************************************************}
procedure PintarDialogo (var Nucleo: tNucleo; mensaje: string; x: word);
begin
   {-----------------------------------------------------------------------------}
   { Borramos la pantalla y escribimos el mensaje del cuadro de dilogo.         }
   {-----------------------------------------------------------------------------}
   M13hBorrarPantalla(COL_NEGRO);

   FuenteEscribir(mensaje, x, 50, COL_TXTDLG, Nucleo.Fuente);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "S" si el ratn est encima.                           }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
      M13hImagenPegar8(Nucleo.Texturas.DlgBotones[BOT_DLG_SI].x,
         Nucleo.Texturas.DlgBotones[BOT_DLG_SI].y,
         Nucleo.Texturas.PosBotones[BOT_SI],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave)
   else
      M13hImagenPegar8(Nucleo.Texturas.DlgBotones[BOT_DLG_SI].x,
         Nucleo.Texturas.DlgBotones[BOT_DLG_SI].y,
         Nucleo.Texturas.PosBotones[BOT_SI_NORMAL],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "No" si el ratn est encima.                           }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
      M13hImagenPegar8(Nucleo.Texturas.DlgBotones[BOT_DLG_NO].x,
         Nucleo.Texturas.DlgBotones[BOT_DLG_NO].y,
         Nucleo.Texturas.PosBotones[BOT_NO],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave)
   else
      M13hImagenPegar8(Nucleo.Texturas.DlgBotones[BOT_DLG_NO].x,
         Nucleo.Texturas.DlgBotones[BOT_DLG_NO].y,
         Nucleo.Texturas.PosBotones[BOT_NO_NORMAL],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);
end;

{********************************************************************************}
{ Funcin que pinta la pantalla del men.                                        }
{********************************************************************************}
procedure PintarMenu (var Nucleo: tNucleo);
begin
   {-----------------------------------------------------------------------------}
   { Pintamos el fondo del men.                                                 }
   {-----------------------------------------------------------------------------}
   M13hImagenPegar(0, 0, Nucleo.Texturas.Menu);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "1 Jugador" si el ratn est encima.                    }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_1JUGADORES]) then
      M13hImagenPegar8(Nucleo.Texturas.MnuBotones[BOT_MNU_1JUGADORES].x,
         Nucleo.Texturas.MnuBotones[BOT_MNU_1JUGADORES].y,
         Nucleo.Texturas.PosBotones[BOT_1JUGADORES],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "2 Jugadores" si el ratn est encima.                  }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_2JUGADORES]) then
      M13hImagenPegar8(Nucleo.Texturas.MnuBotones[BOT_MNU_2JUGADORES].x,
         Nucleo.Texturas.MnuBotones[BOT_MNU_2JUGADORES].y,
         Nucleo.Texturas.PosBotones[BOT_2JUGADORES],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Cargar" si el ratn est encima.                       }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_CARGAR]) then
      M13hImagenPegar8(Nucleo.Texturas.MnuBotones[BOT_MNU_CARGAR].x,
         Nucleo.Texturas.MnuBotones[BOT_MNU_CARGAR].y,
         Nucleo.Texturas.PosBotones[BOT_CARGAR],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Ayuda" si el ratn est encima.                        }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_AYUDA]) then
      M13hImagenPegar8(Nucleo.Texturas.MnuBotones[BOT_MNU_AYUDA].x,
         Nucleo.Texturas.MnuBotones[BOT_MNU_AYUDA].y,
         Nucleo.Texturas.PosBotones[BOT_AYUDA],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Salir" si el ratn est encima.                        }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_SALIR]) then
      M13hImagenPegar8(Nucleo.Texturas.MnuBotones[BOT_MNU_SALIR].x,
         Nucleo.Texturas.MnuBotones[BOT_MNU_SALIR].y,
         Nucleo.Texturas.PosBotones[BOT_SALIR],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);
end;

{********************************************************************************}
{ Funcin que pinta el tablero con la partida actual en curso.                   }
{********************************************************************************}
procedure PintarTablero (var Nucleo: tNucleo);
var
   i, j: integer;
begin
   {-----------------------------------------------------------------------------}
   { Pintamos el fondo.                                                          }
   {-----------------------------------------------------------------------------}
   M13hImagenPegar(0, 0, Nucleo.Texturas.Tablero);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Reiniciar" si el ratn est encima.                    }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_REINICIAR]) then
      M13hImagenPegar8(Nucleo.Texturas.TabBotones[BOT_TAB_REINICIAR].x,
         Nucleo.Texturas.TabBotones[BOT_TAB_REINICIAR].y,
         Nucleo.Texturas.PosBotones[BOT_REINICIAR],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Salvar" si el ratn est encima.                       }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_SALVAR]) then
      M13hImagenPegar8(Nucleo.Texturas.TabBotones[BOT_TAB_SALVAR].x,
         Nucleo.Texturas.TabBotones[BOT_TAB_SALVAR].y,
         Nucleo.Texturas.PosBotones[BOT_SALVAR],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Cargar" si el ratn est encima.                       }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_CARGAR]) then
      M13hImagenPegar8(Nucleo.Texturas.TabBotones[BOT_TAB_CARGAR].x,
         Nucleo.Texturas.TabBotones[BOT_TAB_CARGAR].y,
         Nucleo.Texturas.PosBotones[BOT_CARGAR],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Pasar" si el ratn est encima.                        }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_PASAR]) then
      M13hImagenPegar8(Nucleo.Texturas.TabBotones[BOT_TAB_PASAR].x,
         Nucleo.Texturas.TabBotones[BOT_TAB_PASAR].y,
         Nucleo.Texturas.PosBotones[BOT_PASAR],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Salir" si el ratn est encima.                        }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_SALIR]) then
      M13hImagenPegar8(Nucleo.Texturas.TabBotones[BOT_TAB_SALIR].x,
         Nucleo.Texturas.TabBotones[BOT_TAB_SALIR].y,
         Nucleo.Texturas.PosBotones[BOT_SALIR],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);

   {-----------------------------------------------------------------------------}
   { Pintamos las piezas en sus respectivas posiciones.                          }
   {-----------------------------------------------------------------------------}
   for i := 1 to Nucleo.Juego.MaxFilas do
      for j := 1 to Nucleo.Juego.MaxColum do
         if Nucleo.Juego.Fichas[i, j] <> FICHA_VACIA then
          begin
            M13hImagenPegar8(TAB_SEP_X * (j - 1) + Nucleo.Juego.TabPosX,
               TAB_SEP_Y * (i - 1) + Nucleo.Juego.TabPosY,
               Nucleo.Texturas.PosFichas[Nucleo.Juego.Fichas[i, j]],
               Nucleo.Texturas.Fichas, Nucleo.Texturas.CClave);
          end;

   {-----------------------------------------------------------------------------}
   { Pintamos en caso de estar activado, alguno de los mensajes para el usuario. }
   {-----------------------------------------------------------------------------}
   case Nucleo.Juego.Mensaje of
      MSJ_NINGUNO:
         if Nucleo.Juego.Turno = TURNO_BLANCAS then begin
            FuenteEscribir('Mueven las', 216, 171, COL_TXTMSJ, Nucleo.Fuente);
            FuenteEscribir(' blancas.',  216, 185, COL_TXTMSJ, Nucleo.Fuente);
         end else begin
            FuenteEscribir('Mueven las', 216, 171, COL_TXTMSJ, Nucleo.Fuente);
            FuenteEscribir(' negras.',   216, 185, COL_TXTMSJ, Nucleo.Fuente);
         end;

      MSJ_MOVINV:
       begin
         FuenteEscribir('Movimiento', 216, 171, COL_TXTMSJ, Nucleo.Fuente);
         FuenteEscribir(' invalido!',  216, 185, COL_TXTMSJ, Nucleo.Fuente);
       end;

      MSJ_BLABLOQ:
       begin
         FuenteEscribir('Blancas sin',  216, 171, COL_TXTMSJ, Nucleo.Fuente);
         FuenteEscribir(' movimientos!', 216, 185, COL_TXTMSJ, Nucleo.Fuente);
       end;

      MSJ_NEGBLOQ:
       begin
         FuenteEscribir('Negras sin',   216, 171, COL_TXTMSJ, Nucleo.Fuente);
         FuenteEscribir(' movimientos!', 216, 185, COL_TXTMSJ, Nucleo.Fuente);
       end;

      MSJ_NOPASAR:
       begin
         FuenteEscribir('No puedes', 216, 171, COL_TXTMSJ, Nucleo.Fuente);
         FuenteEscribir(' pasar!',    216, 185, COL_TXTMSJ, Nucleo.Fuente);
       end;
   end;
end;

{********************************************************************************}
{ Funcin que pinta el estado actual del juego.                                  }
{********************************************************************************}
procedure PintarJuego (var Nucleo: tNucleo);
begin
   case Nucleo.Juego.Victoria of
      {--------------------------------------------------------------------------}
      { Pintamos el tablero si todava no hay una victoria.                      }
      {--------------------------------------------------------------------------}
      VICTORIA_NO:
       begin
         PintarTablero(Nucleo);
       end;

      {--------------------------------------------------------------------------}
      { Ganan las blancas.                                                       }
      {--------------------------------------------------------------------------}
      VICTORIA_J1:
       begin
         PintarDialogo(Nucleo, 'Desea jugar otra partida?', 69);

         if Nucleo.Juego.Estado = EST_1JUG then
          begin
            FuenteEscribir('Gana el jugador!', 100, 0, COL_TXTVIC, Nucleo.Fuente);
            PintarLineaH(100, 219, 14, COL_TXTSAL);
          end
         else
          begin
            FuenteEscribir('Gana el jugador 1!', 93, 0, COL_TXTVIC, Nucleo.Fuente);
            PintarLineaH(93, 226, 14, COL_TXTSAL);
          end;
       end;

      {--------------------------------------------------------------------------}
      { Ganan las negras.                                                        }
      {--------------------------------------------------------------------------}
      VICTORIA_J2:
       begin
         PintarDialogo(Nucleo, 'Desea jugar otra partida?', 69);

         if Nucleo.Juego.Estado = EST_1JUG then
          begin
            FuenteEscribir('Gana la maquina!', 100, 0, COL_TXTVIC, Nucleo.Fuente);
            PintarLineaH(100, 219, 14, COL_TXTSAL);
          end
         else
          begin
            FuenteEscribir('Gana el jugador 2!', 93, 0, COL_TXTVIC, Nucleo.Fuente);
            PintarLineaH(93, 226, 14, COL_TXTSAL);
          end;
       end;

      {--------------------------------------------------------------------------}
      { Ganan las negras.                                                        }
      {--------------------------------------------------------------------------}
      JUEGO_TABLAS:
       begin
         PintarDialogo(Nucleo, 'Desea jugar otra partida?', 69);

         FuenteEscribir('Juego en tablas!', 100, 0, COL_TXTVIC, Nucleo.Fuente);
         PintarLineaH(100, 219, 14, COL_TXTSAL);
       end;
   end;
end;

{********************************************************************************}
{ Funcin que pinta los nombres de las partidas guardadas.                       }
{********************************************************************************}
procedure PintarPartidas (var Nucleo: tNucleo);
var
   i: integer;
begin
   for i := 1 to MAX_PARTIDAS do
    begin
      {--------------------------------------------------------------------------}
      { Pintamos primero un rectngulo donde va a ir el nombre.                  }
      {--------------------------------------------------------------------------}
      PintarRectangulo(PART_INITX, PART_INITX + PART_INITW,
         PART_INITY + PART_SEP_Y * (i - 1),
         (PART_INITY + PART_INITH) + PART_SEP_Y * (i - 1), COL_BLANCO);

      {--------------------------------------------------------------------------}
      { Y despus pintamos el nombre dentro de dicho rectngulo.                 }
      {--------------------------------------------------------------------------}
      FuenteEscribir(Nucleo.Partidas.Nombre[i], PART_INITX + 2,
         (PART_INITY + 1) + PART_SEP_Y * (i - 1), COL_BLANCO, Nucleo.Fuente);
    end;
end;

{********************************************************************************}
{ Funcin que pinta la pantalla de salvar la partida.                            }
{********************************************************************************}
procedure PintarSalvar (var Nucleo: tNucleo);
begin
   case Nucleo.Partidas.Estado of
      PART_EST_NORMAL:
       begin
         {-----------------------------------------------------------------------}
         { Borramos la pantalla y escribimos el ttulo de la ventana.            }
         {-----------------------------------------------------------------------}
         M13hBorrarPantalla(COL_NEGRO);

         FuenteEscribir('Salvar juego', 118, 0, COL_TXTSAL, Nucleo.Fuente);
         PintarLineaH(118, 202, 14, COL_TXTSAL);

         {-----------------------------------------------------------------------}
         { Pintamos los nombres de las partidas.                                 }
         {-----------------------------------------------------------------------}
         PintarPartidas(Nucleo);
       end;
      PART_EST_AVISO:
         PintarDialogo(Nucleo, 'Desea guardar sobre esa partida?', 44);
      PART_EST_METER:
       begin
         {-----------------------------------------------------------------------}
         { Borramos la pantalla y escribimos el ttulo de la ventana.            }
         {-----------------------------------------------------------------------}
         M13hBorrarPantalla(COL_NEGRO);

         FuenteEscribir('Salvar juego', 118, 0, COL_TXTSAL, Nucleo.Fuente);
         PintarLineaH(118, 202, 14, COL_TXTSAL);

         {-----------------------------------------------------------------------}
         { Y escribimos el texto actual del nombre de la partida.                }
         {-----------------------------------------------------------------------}
         FuenteEscribir('Introduzca el nombre de la partida:', 10, 86, COL_TXTSAL, Nucleo.Fuente);
         FuenteEscribir(Nucleo.Partidas.Auxcad, 10, 100, COL_TXTSAL, Nucleo.Fuente);
       end;
   end;
end;

{********************************************************************************}
{ Funcin que pinta la pantalla de cargar la partida.                            }
{********************************************************************************}
procedure PintarCargar (var Nucleo: tNucleo);
begin
   case Nucleo.Partidas.Estado of
      PART_EST_NORMAL:
       begin
         {-----------------------------------------------------------------------}
         { Borramos la pantalla y escribimos el ttulo de la ventana.            }
         {-----------------------------------------------------------------------}
         M13hBorrarPantalla(COL_NEGRO);

         FuenteEscribir('Cargar juego', 118, 0, COL_TXTCAR, Nucleo.Fuente);
         PintarLineaH(118, 202, 14, COL_TXTSAL);

         {-----------------------------------------------------------------------}
         { Pintamos los nombres de las partidas.                                 }
         {-----------------------------------------------------------------------}
         PintarPartidas(Nucleo);
       end;
      PART_EST_AVISO:
         PintarDialogo(Nucleo, 'Desea cargar esa partida?', 69);
   end;
end;

{********************************************************************************}
{ Funcin que pinta la ayuda del juego.                                          }
{********************************************************************************}
procedure PintarAyuda (var Nucleo: tNucleo);
begin {320/2 = 160}
   {-----------------------------------------------------------------------------}
   { Primera pgina de ayuda.                                                    }
   {-----------------------------------------------------------------------------}
   M13hBorrarPantalla(COL_NEGRO);

   FuenteEscribir('Ayuda del juego', 107, 0, COL_TXTTIT, Nucleo.Fuente);
   PintarLineaH(107, 212, 14, COL_TXTTIT);

   {             ('*********************************************')}
   FuenteEscribir('El Othello es un juego de cierta complejidad', 3, 16, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('cuyo objetivo final es ser quien ms fichas',  3, 30, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('de su color tenga en el tablero. Para ello',   3, 44, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('empezamos en un tablero con dos fichas cada',  3, 58, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('por bando, colocadas en el centro. El',        3, 72, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('tablero podr tener unas dimensiones desde',   3, 86, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('2x2, hasta 10x10 como mximo (tambin se',     3, 100, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('permitirn dimensiones como 3x4). El juego',   3, 114, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('terminar cuando se llene el tablero de',      3, 128, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('fichas, o ninguno de los dos pueda mover.',    3, 142, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('Cuando no se puede mover el jugador, tendr',  3, 156, COL_TXTAYU, Nucleo.Fuente);

   FuenteEscribir('Pulse una tecla para continuar...', 3, 184, COL_TXTCON, Nucleo.Fuente);
   M13hActualizarPantalla; readkey;

   {-----------------------------------------------------------------------------}
   { Segunda pgina de ayuda.                                                    }
   {-----------------------------------------------------------------------------}
   M13hBorrarPantalla(COL_NEGRO);

   FuenteEscribir('Ayuda del juego', 107, 0, COL_TXTTIT, Nucleo.Fuente);
   PintarLineaH(107, 212, 14, COL_TXTTIT);

   {             ('*********************************************')}
   FuenteEscribir('que pulsar el botn de pasar. Para obtener',   3, 16, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('fichas tendremos que situar una ficha al',     3, 30, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('lado de otra del oponente, pero cumpliendo',   3, 44, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('la condicin de que en la horizontal, o la',   3, 58, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('vertical, o alguna de las diagonales, exista', 3, 72, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('una ficha nuestra pegada a las enemigas:',     3, 86, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('                   ',           3, 100, COL_TXTEJE, Nucleo.Fuente);
   FuenteEscribir('   XX Mueven XX Mueven XX',           3, 114, COL_TXTEJE, Nucleo.Fuente);
   FuenteEscribir('   OOO   ->   XXO   ->   OOOO',           3, 128, COL_TXTEJE, Nucleo.Fuente);
   FuenteEscribir('   X las X  XX las O  XX',           3, 142, COL_TXTEJE, Nucleo.Fuente);
   FuenteEscribir('                   ',           3, 156, COL_TXTEJE, Nucleo.Fuente);

   FuenteEscribir('Pulse una tecla para continuar...', 3, 184, COL_TXTCON, Nucleo.Fuente);
   M13hActualizarPantalla; readkey;

   {-----------------------------------------------------------------------------}
   { Tercera pgina de ayuda.                                                    }
   {-----------------------------------------------------------------------------}
   M13hBorrarPantalla(COL_NEGRO);

   FuenteEscribir('Ayuda del juego', 107, 0, COL_TXTTIT, Nucleo.Fuente);
   PintarLineaH(107, 212, 14, COL_TXTTIT);

   {             ('*********************************************')}
   FuenteEscribir('La interfaz del juego es bastante sencilla,',   3, 16, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('se emplea el ratn para pulsar las casillas',   3, 30, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('del tablero, y soltar as una ficha. Y esto',   3, 44, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('es todo, buena suerte con el juego y que te.',  3, 58, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('sea leve jugar al Othello.',                    3, 72, COL_TXTAYU, Nucleo.Fuente);


   FuenteEscribir('Grficos y programacin por Gorka Surez',      3, 142, COL_TXTAYU, Nucleo.Fuente);
   FuenteEscribir('Garca, alias "Alexander Lessman".',            3, 156, COL_TXTAYU, Nucleo.Fuente);

   FuenteEscribir('Pulse una tecla para volver al men...', 3, 184, COL_TXTCON, Nucleo.Fuente);
   M13hActualizarPantalla; readkey;
end;

{********************************************************************************}
{ Funcin que pinta el cuadro de dilogo de salir del juego.                     }
{********************************************************************************}
procedure PintarSalir1 (var Nucleo: tNucleo);
begin
   PintarDialogo(Nucleo, 'Desea salir del juego?', 79);
end;

{********************************************************************************}
{ Funcin que pinta el cuadro de dilogo de salir de la partida.                 }
{********************************************************************************}
procedure PintarSalir2 (var Nucleo: tNucleo);
begin
   PintarDialogo(Nucleo, 'Desea salir de la partida?', 65);
end;

{********************************************************************************}
{ Funcin que pinta el cuadro de dilogo de reiniciar la partida.                }
{********************************************************************************}
procedure PintarReiniciar (var Nucleo: tNucleo);
begin
   PintarDialogo(Nucleo, 'Desea reiniciar el juego?', 69);
end;

{********************************************************************************}
{ Funcin que pinta la pantalla para elegir el tamao del tablero.               }
{********************************************************************************}
procedure PintarElegirTam (var Nucleo: tNucleo);
var
   i: byte;
   num: string;
begin
   {-----------------------------------------------------------------------------}
   { Borramos la pantalla y escribimos el mensaje del cuadro de dilogo.         }
   {-----------------------------------------------------------------------------}
   M13hBorrarPantalla(COL_NEGRO);

   {-----------------------------------------------------------------------------}
   { Pintamos los botones para elegir el ancho del tablero.                      }
   {-----------------------------------------------------------------------------}
   FuenteEscribir('Elije un ancho', TAMBOT_TXT1X, TAMBOT_TXT1Y, COL_TXTNOM, Nucleo.Fuente);

   for i := TAMBOT_COLINI to TAMBOT_COLFIN do begin
      if Nucleo.Juego.MaxColum = i then begin
         M13hImagenPegar8(TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X, TAMBOT_INI1Y,
            Nucleo.Texturas.PosTamBot[TAMBOT_PULSADO], Nucleo.Texturas.Fichas,
            Nucleo.Texturas.CClave);
      end else begin
         M13hImagenPegar8(TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X, TAMBOT_INI1Y,
            Nucleo.Texturas.PosTamBot[TAMBOT_SINPULSAR], Nucleo.Texturas.Fichas,
            Nucleo.Texturas.CClave);
      end;

      Str(i, num);
      if i < 10 then
         FuenteEscribir(num, TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X + TAMBOT_SEPTX,
            TAMBOT_INI1Y + TAMBOT_SEPTY, COL_TXTNOM, Nucleo.Fuente)
      else
         FuenteEscribir(num, TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X + (TAMBOT_SEPTX - 4),
            TAMBOT_INI1Y + TAMBOT_SEPTY, COL_TXTNOM, Nucleo.Fuente);
   end;
   
   {-----------------------------------------------------------------------------}
   { Pintamos los botones para elegir el alto del tablero.                       }
   {-----------------------------------------------------------------------------}
   FuenteEscribir('Elije un alto', TAMBOT_TXT2X, TAMBOT_TXT2Y, COL_TXTNOM, Nucleo.Fuente);
   
   for i := TAMBOT_COLINI to TAMBOT_COLFIN do begin
      if Nucleo.Juego.MaxFilas = i then begin
         M13hImagenPegar8(TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X, TAMBOT_INI2Y,
            Nucleo.Texturas.PosTamBot[TAMBOT_PULSADO], Nucleo.Texturas.Fichas,
            Nucleo.Texturas.CClave);
      end else begin
         M13hImagenPegar8(TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X, TAMBOT_INI2Y,
            Nucleo.Texturas.PosTamBot[TAMBOT_SINPULSAR], Nucleo.Texturas.Fichas,
            Nucleo.Texturas.CClave);
      end;

      Str(i, num);
      if i < 10 then
         FuenteEscribir(num, TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X + TAMBOT_SEPTX,
            TAMBOT_INI2Y + TAMBOT_SEPTY, COL_TXTNOM, Nucleo.Fuente)
      else
         FuenteEscribir(num, TAMBOT_INITX + (i - TAMBOT_COLINI) * TAMBOT_SEP_X + (TAMBOT_SEPTX - 4),
            TAMBOT_INI2Y + TAMBOT_SEPTY, COL_TXTNOM, Nucleo.Fuente);
   end;
end;

{********************************************************************************}
{ Funcin que pinta la pantalla para elegir el tamao del tablero.               }
{********************************************************************************}
procedure PintarElegirNombreJ1 (var Nucleo: tNucleo);
begin
   {-----------------------------------------------------------------------------}
   { Borramos la pantalla y escribimos el mensaje del cuadro de dilogo.         }
   {-----------------------------------------------------------------------------}
   M13hBorrarPantalla(COL_NEGRO);

   FuenteEscribir('Nueva partida', 118, 0, COL_TXTNOM, Nucleo.Fuente);
   PintarLineaH(118, 210, 14, COL_TXTNOM);

   {-----------------------------------------------------------------------}
   { Y escribimos el texto actual del nombre de la partida.                }
   {-----------------------------------------------------------------------}
   FuenteEscribir('Introduzca el nombre del jugador 1:', 10, 86, COL_TXTNOM, Nucleo.Fuente);
   FuenteEscribir(Nucleo.Juego.NombreJ1, 10, 100, COL_TXTNOM, Nucleo.Fuente);
end;

{********************************************************************************}
{ Funcin que pinta la pantalla para elegir el tamao del tablero.               }
{********************************************************************************}
procedure PintarElegirNombreJ2 (var Nucleo: tNucleo);
begin
   {-----------------------------------------------------------------------------}
   { Borramos la pantalla y escribimos el mensaje del cuadro de dilogo.         }
   {-----------------------------------------------------------------------------}
   M13hBorrarPantalla(COL_NEGRO);

   FuenteEscribir('Nueva partida', 118, 0, COL_TXTNOM, Nucleo.Fuente);
   PintarLineaH(118, 210, 14, COL_TXTNOM);

   {-----------------------------------------------------------------------}
   { Y escribimos el texto actual del nombre de la partida.                }
   {-----------------------------------------------------------------------}
   FuenteEscribir('Introduzca el nombre del jugador 2:', 10, 86, COL_TXTNOM, Nucleo.Fuente);
   FuenteEscribir(Nucleo.Juego.NombreJ2, 10, 100, COL_TXTNOM, Nucleo.Fuente);
end;

{********************************************************************************}
{ Funcin que pinta la pantalla para elegir el tamao del tablero.               }
{********************************************************************************}
procedure PintarElegirDificultad (var Nucleo: tNucleo);
begin
   {-----------------------------------------------------------------------------}
   { Borramos la pantalla y escribimos el mensaje del cuadro de dilogo.         }
   {-----------------------------------------------------------------------------}
   M13hBorrarPantalla(COL_NEGRO);

   FuenteEscribir('Elija la dificultad', DIFBOT_TITX, DIFBOT_TITY, COL_TXTDLG, Nucleo.Fuente);

   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Fcil" si el ratn est encima.                        }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.PosDifBot[BOT_DIF_FACIL]) then
      M13hImagenPegar8(Nucleo.Texturas.PosDifBot[BOT_DIF_FACIL].x,
         Nucleo.Texturas.PosDifBot[BOT_DIF_FACIL].y,
         Nucleo.Texturas.PosBotones[BOT_FACIL],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave)
   else
      M13hImagenPegar8(Nucleo.Texturas.PosDifBot[BOT_DIF_FACIL].x,
         Nucleo.Texturas.PosDifBot[BOT_DIF_FACIL].y,
         Nucleo.Texturas.PosBotones[BOT_FACIL_SP],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);
         
   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Normal" si el ratn est encima.                       }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.PosDifBot[BOT_DIF_NORMAL]) then
      M13hImagenPegar8(Nucleo.Texturas.PosDifBot[BOT_DIF_NORMAL].x,
         Nucleo.Texturas.PosDifBot[BOT_DIF_NORMAL].y,
         Nucleo.Texturas.PosBotones[BOT_NORMAL],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave)
   else
      M13hImagenPegar8(Nucleo.Texturas.PosDifBot[BOT_DIF_NORMAL].x,
         Nucleo.Texturas.PosDifBot[BOT_DIF_NORMAL].y,
         Nucleo.Texturas.PosBotones[BOT_NORMAL_SP],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);
         
   {-----------------------------------------------------------------------------}
   { Iluminamos el botn "Difcil" si el ratn est encima.                      }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.PosDifBot[BOT_DIF_DIFICIL]) then
      M13hImagenPegar8(Nucleo.Texturas.PosDifBot[BOT_DIF_DIFICIL].x,
         Nucleo.Texturas.PosDifBot[BOT_DIF_DIFICIL].y,
         Nucleo.Texturas.PosBotones[BOT_DIFICIL],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave)
   else
      M13hImagenPegar8(Nucleo.Texturas.PosDifBot[BOT_DIF_DIFICIL].x,
         Nucleo.Texturas.PosDifBot[BOT_DIF_DIFICIL].y,
         Nucleo.Texturas.PosBotones[BOT_DIFICIL_SP],
         Nucleo.Texturas.Botones, Nucleo.Texturas.CClave);
end;

{********************************************************************************}
{ Funcin principal que gestiona el pintado de la pantalla en el juego.          }
{********************************************************************************}
procedure PintarPantalla (var Nucleo: tNucleo);
begin
   {-----------------------------------------------------------------------------}
   { Seleccionamos, dependiendo del estado del juego, una funcin para pintar    }
   { en cada uno de los estados del juego.                                       }
   {-----------------------------------------------------------------------------}
   case Nucleo.Juego.Estado of
      EST_MENU:       PintarMenu(Nucleo);
      EST_1JUG:       PintarJuego(Nucleo);
      EST_2JUG:       PintarJuego(Nucleo);
      EST_SALVAR1:    PintarSalvar(Nucleo);
      EST_SALVAR2:    PintarSalvar(Nucleo);
      EST_CARGAR1:    PintarCargar(Nucleo);
      EST_CARGAR2:    PintarCargar(Nucleo);
      EST_CARGAR3:    PintarCargar(Nucleo);
      EST_AYUDA1:     PintarAyuda(Nucleo);
      EST_SALIR1:     PintarSalir1(Nucleo);
      EST_SALIR2:     PintarSalir2(Nucleo);
      EST_SALIR3:     PintarSalir2(Nucleo);
      EST_REINICIAR1: PintarReiniciar(Nucleo);
      EST_REINICIAR2: PintarReiniciar(Nucleo);
      EST_ELITAM1:    PintarElegirTam(Nucleo);
      EST_ELINOM1J1:  PintarElegirNombreJ1(Nucleo);
      EST_ELIDIFICUL: PintarElegirDificultad(Nucleo);
      EST_ELITAM2:    PintarElegirTam(Nucleo);
      EST_ELINOM2J1:  PintarElegirNombreJ1(Nucleo);
      EST_ELINOM2J2:  PintarElegirNombreJ2(Nucleo);
   end;

   {-----------------------------------------------------------------------------}
   { En caso de que no estemos en la pantalla de ayuda, pintamos el cursor, y    }
   { luego actualizamos el contenido de la pantalla.                             }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Estado <> EST_AYUDA1) then
    begin
      if Nucleo.Raton.bi then
         M13hImagenPegar8(Nucleo.Raton.x, Nucleo.Raton.y,
            Nucleo.Texturas.PosCursor[CUR_COGER],
            Nucleo.Texturas.Cursor, Nucleo.Texturas.CClave)
      else
         M13hImagenPegar8(Nucleo.Raton.x, Nucleo.Raton.y,
            Nucleo.Texturas.PosCursor[CUR_NORMAL],
            Nucleo.Texturas.Cursor, Nucleo.Texturas.CClave);

      M13hActualizarPantalla;
    end;
end;

{********************************************************************************}
end.
{********************************************************************************}